function BeneCast:OnInitialize()

end

function BeneCast:OnEnable()

end

function BeneCast:OnDisable()

end

function BeneCast:GetProfile(pf, unit)
	if not BeneCastProfiles then
		return;
	end
	
	if not BeneCastProfiles[profileid] then
		self:ResetProfile(profileid);
	end
	
	if not BeneCastProfiles[profileid].SnapTo then
		BeneCastProfiles[profileid].SnapTo = {};
	end
	
	if unit and not BeneCastProfiles[profileid].SnapTo[unit] then
		BeneCastProfiles[profileid].SnapTo[unit] = {};
	end
	return BeneCastProfiles[profileid], BeneCastProfiles[profileid].SnapTo[unit];
end

function BeneCast:GetCurrentProfileID()
	if wv.ConsiderInCombat then
		return 'combat';
	else
		return 'ooc';
	end
end

-- Function to resize the buttons of a profile
function BeneCast:ResizeButtonsForProfile(pf)

	if BeneCast.Panels[pf] == nil then
		BeneCast.Panels[pf] = {};
	end
	
	-- Resize the buttons of every panel
	for unit, panel in pairs (BeneCast.Panels[pf]) do
		panel:ResizeButtons(i);
	end
	
end

-- Resets data tables
function BeneCast:ResetData()

	-- Clear the BC_spell_data table
	for type in pairs (BC_spell_data) do
		_ClearTable(wv.SpellData);
	end
	
	-- Reload the spell data
	self:LoadSpellData();
	
end

-- *****************************************************************************
-- BeneCast Spell Loading function
-- *****************************************************************************

--Sort two spells by order in BeneCast_SpellTypes_Sorting
function BeneCast_GetSpellPos(spelltype, spellname)
	local pos = 0;
	for i, spellname2 in pairs (BeneCast_SpellTypes_Sorting[BC_class][spelltype]) do
		if spellname2 == spellname then
			pos = i;
			break;
		end
	end
	return pos;
end

function BeneCast_SortTwoSpells(a,b)
	if (a.name ~= b.name) then
		return BeneCast_GetSpellPos(a.spelltype,a.name) < BeneCast_GetSpellPos(b.spelltype,b.name);
	elseif a.rankno and b.rankno then
		return a.rankno < b.rankno;
	else
		return a.id < b.id; -- Lower ranks are placed before higher ranks in the spellbook
	end
end

-- Fill in the BC_spell_data table by iterating over spells in spellbook
-- and parsing tool tips for info. Thanks Danboo!
function BeneCast_LoadSpellData()
	PutDebugMsg('Parsing Spell Data from spellbook');
	
	BC_spell_data = {}; 
	
	-- Iterator to move through the spellbook
	local i = 1;

	while true do
		
		-- Spell at position i in the player's spellbook
		local spell_name, spell_rank = GetSpellName(i, SpellBookFrame.bookType);
		
		-- If there is no spell at i, break out of the loop
		if not spell_name then
			do break end
		end
		
		-- Load up the spell_type based on the spell name and player class
		local spell_type = BeneCast_SpellTypes[BC_class][spell_name];

		-- If the spell_type exists do stuff
		if ( spell_type ) then
						
			-- If there isn't anything in BC_spell_data[spell_type] yet then create a table for it
			if ( not BC_spell_data[spell_type] ) then
				BC_spell_data[spell_type] = {};
				BC_spell_data[spell_type]['spells'] = {};
			end
			
			local rankno;
			_, _, rankno = string.find(spell_rank,'%d+'); -- Find any string of numbers in the rank
			if rankno then
				rankno = tonumber(rankno);
			end
			
			local entry;
			if spell_rank and spell_rank ~= '' then
				entry = {
					name = (spell_name),
					rank = (spell_rank),
					rankno = rankno,
					spelltype = spell_type,
					id = i
				};
			-- If there is no rank, don't put a spell rank into the entry
			else
				entry = {
					name = (spell_name),
					spelltype = spell_type,
					id = i
				};
			end
			
			-- Insert the entry into the appropriate table
			table.insert(BC_spell_data[spell_type]['spells'], entry);

		end

		i = i + 1;

	end
	
	-- Sort the spells by mana cost within each spell type
	for type in pairs (BC_spell_data) do
		table.sort(BC_spell_data[type]['spells'], BeneCast_SortTwoSpells);
	end
	
	-- If the spell_type does not have a texture associated with it set it
	for type in pairs (BC_spell_data) do
		local texture;
		-- Make sure the texture is for the last entry in the table
		local i = table.getn(BC_spell_data[type]['spells']);
		BC_spell_data[type].spellcount = i;
		texture = GetSpellTexture(BC_spell_data[type]['spells'][i].id, SpellBookFrame.bookType);
		BC_spell_data[type].texture = texture;
		BC_spell_data[type].type = type;
	end
	
end


-- *****************************************************************************
-- BeneCast spell choose functions
-- *****************************************************************************

-- Interpolates the health of the given unit (used for those outside the party). Thanks Danboo!
-- Returns an estimated health for a target outside the party
function BeneCast_UnitInterpolateHealth(unit, health_percent)
	
	local class      = UnitClass(unit);
	if class == nil then
		class = BENECAST_STRINGS.CLASS_WARRIOR;
	end
	local level      = UnitLevel(unit);
	local slope      = ( BC_class_health[class]['60'] - BC_class_health[class]['1'] ) / 59;
	local health_max = level * slope + BC_class_health[class]['1'];
	local health     = health_max * health_percent / 100;
	return floor(health), floor(health_max);
	
end

-- *****************************************************************************
-- BeneCast Slash Command Handler
-- *****************************************************************************

-- Slash command handler
-- The only slash command is to show the config GUI
function BeneCast_SlashCommandHandler(msg)

	if BeneCastSetup.currentid == nil then
		BeneCastSetup.currentid = 1;
	end
	local profileid = BeneCastSetup.currentid;
	
	if string.find(msg,'debug') then
		BeneCastSetup['Debug'] = not BeneCastSetup['Debug'];
		PutMsg('command: ' .. msg .. ' , debug is now '..(BeneCastSetup['Debug'] and 'on' or 'off'));
	elseif string.find(msg,'peekb (.+)') then
		local varname;
		_, _, varname = string.find(msg,'peekb (.+)');
		if varname then
			PutMsg('command: ' .. msg .. ' , BeneCastProfiles[' .. profileid .. ']['..varname..'] = ' .. ( (not BeneCastProfiles[profileid][varname]) and 'nil' or BeneCastProfiles[profileid][varname] ));
		else
			PutMsg('command: ' .. msg .. ' , invalid parameter');
		end
	elseif string.find(msg,'peek (.+) (.+)') then
		local varname, varname2;
		_, _, varname, varname2 = string.find(msg,'peek (.+) (.+)');
		if varname and varname2 then
			PutMsg('command: '..msg..' , BeneCastProfiles[' .. profileid .. ']['..varname..'][' .. varname2 .. '] = '..( (not BeneCastProfiles[profileid][varname][varname2]) and 'nil' or BeneCastProfiles[profileid][varname] ) );
		else
			PutMsg('command: ' .. msg .. ' , invalid parameter');
		end
	elseif string.find(msg,'peek (.+)') then
		local varname;
		_, _, varname = string.find(msg,'peek (.+)');
		if varname then
			if BeneCastProfiles[profileid][varname] then
				PutMsg('command : ' .. msg .. ' , BeneCastProfiles[' .. profileid .. '][' .. varname ..
				  '] = ' .. BeneCastProfiles[profileid][varname]);
			else
				PutMsg('command : ' .. msg .. ' , BeneCastProfiles[' .. profileid .. '][' .. varname .. 
				  '] = nil or not found');
			end
		else
			PutMsg('command : ' .. msg .. ' , invalid parameter');
		end
	elseif string.find(msg,'snapto2 (.+)') then
		local addonpar;
		_, _, addonpar = string.find(msg,'snapto2 (.+)');
		if addonpar then
			PutMsg('trying to snapto ' .. addonpar);
			SetBeneCastSnapTo2(addonpar);
		else
			PutMsg('snapto2 had no value after it');
		end
	elseif string.find(msg,'snapto (.+)') then
		local addonname = nil;
		_, _, addonname = string.find(msg,'snapto (.+)');
		if addonname then
			PutMsg('trying to snapto ' .. addonname);
			SetBeneCastSnapTo(addonname);
		else
			PutMsg('snapto had no addonname after it');
		end
	elseif string.find(msg,'firstaid (.+)') then
		local item;
		_, _, item = string.find(msg,'firstaid (.+)');
		if item then
			BeneCastSetup.firstaiditem = item;
			PutMsg('setting First Aid item to ' .. BeneCastSetup.firstaiditem);
			for i in pairs (BC_targets) do
				BeneCast_UpdateButtons(i);
			end
		else
			PutMsg('firstaid had no value after it');
		end
	elseif string.find(msg,'shadowcancel') then
		BeneCastSetup['ShadowCancel'] = not BeneCastSetup['ShadowCancel'];
		PutMsg('command: ' .. msg .. ' , shadowcancel is now '..(BeneCastSetup['ShadowCancel'] and 'on' or 'off'));
	else
		BeneCastOptionFrameToggle();
	end
	
end

-- *****************************************************************************
-- BeneCast Party Notification functions
-- *****************************************************************************

-- Function for Party Notification
-- Modified SpeakSpell by Danboo. Thanks Danboo!
-- Returns true if a notification message was sent
function BeneCast_PartyNotify(target, spell, ismax, duration)

	if BeneCastSetup.currentid == nil then
		BeneCastSetup.currentid = 1;
	end
	local profileid = BeneCastSetup.currentid;
	if BeneCastProfiles[profileid] == nil then
	  BeneCastProfiles[profileid] = {};
	end
	
	if ( not BeneCastProfiles[profileid].Notification ) then
		return nil;
	end
	
	-- If there is no duration, give a duration of 0
	if ( not duration ) then
		duration = 0;
	end
	
	-- Don't show time if notifytime is nil or duration is 0
	local time = nil;
	if ( duration > 0 and BeneCastProfiles[profileid].NotifyTime ) then
		time = true;
	end
	
	-- Don't show rank if spell.rank is nil
	local rank = nil;
	if ( spell.rank and spell.rank ~= 'Rank 0' ) then
		rank = true;
	else
		spell.rank = 'Rank 0';
	end
	
	-- Figure out the spelltype for display purposes
	local type = BeneCast_SpellTypes[BC_class][spell.name];
	
	if type == nil then 
		PutDebugMsg('Unknown spell class ' .. BC_class .. ', spell ' .. spell.name);
		return nil;
	end
	
	local healamount_text = '';
	
	-- If the type is a heal prepare the healamount_text
	if ( string.find(type,'efficient') or string.find(type,'emergency') or string.find(type,'instant') or 
	     type == 'group' or type == 'group2' ) then
		healamount_text = ' (';
		if spell.max_heal then
			healamount_text = healamount_text .. spell.max_heal;
		else
			healamount_text = healamount_text .. '???';
		end
		if BC_plusheal then
			if BC_plusheal ~= 0 then
				healamount_text = healamount_text .. '+' .. BC_plusheal .. ')';
			else
				healamount_text = healamount_text .. ')';
			end
		else
			healamount_text = healamount_text .. ')';
		end
	end

	duration = string.format('%.1f', duration / 1000);

	local message_text;
	
	-- Set the message text based on what options are checked
	if ( rank and BeneCastProfiles[profileid].NotifyRank and time ) then
		message_text = BENECAST_STRINGS.NOTIFY_ALL;
	elseif ( rank and BeneCastProfiles[profileid].NotifyRank ) then
		message_text = BENECAST_STRINGS.NOTIFY_RANK;
	elseif ( time ) then
		message_text = BENECAST_STRINGS.NOTIFY_TIME;
	else
		message_text = BENECAST_STRINGS.NOTIFY;
	end
	
	message_text = string.gsub(message_text, '%%s', spell.name);
	message_text = string.gsub(message_text, '%%d', duration);
	message_text = string.gsub(message_text, '%%r', spell.rank);
	message_text = message_text .. healamount_text; --WINTROW.6
	
	-- First determine what type of spell it is
	local heal, cure, buff, res;
	if ( string.find(type,'efficient') or string.find(type,'emergency') or 
	     string.find(type,'instant') or type == 'group' or type == 'group2' ) then
		heal = true;
	elseif ( type == 'poison' or type == 'disease' or type == 'disease2' or 
	         type == 'magic' or type == 'curse' ) then
		cure = true;
	elseif ( type == 'res' ) then
		res = true;
	else
		buff = true;
	end
	local rare;
	if type == 'res' or 
	   ( BC_class == BENECAST_STRINGS.CLASS_DRUID and type == 'buff3' ) or
	   ( BC_class == BENECAST_STRINGS.CLASS_HUNTER and type == 'selfbuff8' ) or
	   ( BC_class == BENECAST_STRINGS.CLASS_ROGUE and type == 'selfbuff5' ) or
	   ( BC_class == BENECAST_STRINGS.CLASS_WARRIOR and ( type == 'selfbuff1' or type == 'selfbuff4') ) or
	   ( BC_class == BENECAST_STRINGS.CLASS_PALADIN and ( type == 'partybuff3' or type == 'loh' ) ) then
		rare = true;
	end

	local channels_sent = {};
	
	for i, values in pairs (BeneCastOptionDropDowns) do
		local enable = values.enable;
		local ok = false;
		local cvar = values.cvar;
		local channel = BeneCastProfiles[profileid][cvar];
		-- Make sure the same channel only gets 1 msg
		if not channels_sent[channel] then
			-- Check if the notification is enabled
			if BeneCastProfiles[profileid][enable] then
				-- And the type of spell corresponds with the type of notification
				if ( enable == 'SelfCasts' ) and ( target == 'player' ) then
					ok = true;
				elseif ( enable == 'NotifyHeal' ) and heal then
					ok = true;
				elseif ( enable == 'NotifyCure') and buff then
					ok = true;
				elseif ( enable == 'NotifyRare' ) and rare then
					ok = true;
				elseif ( enable == 'NotifyRes' ) and res then
					ok = true;
				elseif ( enable == 'NotifyBuff' ) and buff then
					ok = true;
				end
			end
		end
		if ok then
			local msg_text = message_text;
		
			if channel == 'WHISPER' then
				-- If notifying the target, address them as 'you'
				msg_text = string.gsub(msg_text, '%%t', BENECAST_STRINGS.NOTIFY_YOU);
			-- In laggy environments if the player's target was deselected while casting it does not have a name
			elseif ( UnitExists(target) ) then
				local name = UnitName(target);
				msg_text = string.gsub(msg_text, '%%t', name);
			else
				msg_text = string.gsub(msg_text, '%%t', BENECAST_STRINGS.NOTIFY_UNKNOWN_ENTITY);
			end
		
			-- Send the message on the appropriate channel
			if ( channel == 'SELF' ) then
				PutMsg(msg_text);
			elseif ( channel == 'PARTY' and UnitExists('party1') ) then
				SendChatMessage(msg_text, 'PARTY');
			elseif ( channel == 'SAY' ) then
				SendChatMessage(msg_text, 'SAY');
			elseif ( channel == 'RAID' and UnitExists('raid1') ) then
				SendChatMessage(msg_text, 'RAID');
			elseif ( channel == 'WHISPER' and target ~= 'player' and UnitIsPlayer(target) ) then
				local name = UnitName(target);
				SendChatMessage(msg_text, 'WHISPER', nil, name);
			elseif ( channel == 'USER' ) then
				SendChatMessage(msg_text, 'CHANNEL', nil, GetChannelName(BeneCastProfiles[profileid].UserChannel));
			-- Thnx to Shimavak for the emote-code
			elseif ( channel == 'EMOTE' ) then 
				SendChatMessage(BENECAST_STRINGS.NOTIFY_EMOTE_PRE .. string.sub(msg_text,2), 'EMOTE');
			end
				
			-- Make sure the same channel doesn't get >1 msg
			channels_sent[channel] = true;
		end
	end
	
	-- Message sent, return true
	return true;

end

-- Function for Party Notification on failures and interruptions
-- Modified SpeakSpell by Danboo. Thanks Danboo!
function BeneCast_PartyNotifyFail(event)

	local message_text;
	
	if BeneCastSetup.currentid == nil then
		BeneCastSetup.currentid = 1;
	end
	local profileid = BeneCastSetup.currentid;
	if BeneCastProfiles[profileid] == nil then
	  BeneCastProfiles[profileid] = {};
  end
	
	if BeneCastProfiles[profileid].NotifyFailure then

		-- Set the message text based on what event called PartyNotifyFail
		-- SPELLCAST_CANCELED is really SPELLCAST_INTERRUPTED if the interruption was caused by the player
		-- To know if interruptions are caused by the player, SPELLCAST_STOP is called just before SPELLCAST_INTERRUPTED
		if event == 'SPELLCAST_FAILED' then
			message_text = BENECAST_STRINGS.NOTIFY_FAILED;
		elseif event == 'SPELLCAST_INTERRUPTED' then
			message_text = BENECAST_STRINGS.NOTIFY_INTERRUPTED;
		elseif event == 'SPELLCAST_CANCELED' then
			message_text = BENECAST_STRINGS.NOTIFY_CANCELED;
		end
		
		local channel = BeneCastProfiles[profileid].NotifyFailureChannel;
	
		-- Send the message on the appropriate channel
		if ( channel == 'SELF' ) then
			PutMsg(msg_text);
		elseif ( channel == 'PARTY' and UnitExists('party1') ) then
			SendChatMessage(msg_text, 'PARTY');
		elseif ( channel == 'SAY' ) then
			SendChatMessage(msg_text, 'SAY');
		elseif ( channel == 'RAID' and UnitExists('raid1') ) then
			SendChatMessage(msg_text, 'RAID');
		elseif ( channel == 'WHISPER' and target ~= 'player' and UnitIsPlayer(target) ) then
			local name = UnitName(target);
			SendChatMessage(msg_text, 'WHISPER', nil, name);
		elseif ( channel == 'USER' ) then
			SendChatMessage(msg_text, 'CHANNEL', nil, GetChannelName(BeneCastProfiles[profileid].UserChannel));
		-- Thnx to Shimavak for the emote-code
		elseif ( channel == 'EMOTE' ) then 
			SendChatMessage(message_text, 'EMOTE');
		end
		
	end

end

-- *****************************************************************************
-- BeneCastButton functions
-- *****************************************************************************

function BeneCast_GetUnitIDFromPanelID(id)
	local target = nil;
	if ( id >= 6 and id <= 45 ) or ( id >= 51 and id <= 90 ) then
		if BENECAST_RAID_PANELS2[id] then
			target = BENECAST_RAID_PANELS2[id]['unitid'];
		end
	else
		target = BC_targets[id];
	end
	return target;
end

-- Make a tooltip for the BeneCastButton based on the options set
function BeneCastButton_OnEnter()

	local id = this:GetParent():GetID();
	local profileid = BeneCast_ProfileForPanelID(id);

	if ( this.spelltype == 'assist' ) then
		GameTooltip:SetOwner(this, 'ANCHOR_BOTTOMRIGHT');
		GameTooltip:SetText(BENECAST_STRINGS.TEXT_ASSIST);
	elseif ( BeneCastProfiles[profileid].ShowTooltips and not BeneCastProfiles[profileid].ShowTooltipsName  ) then
		if this.spellid then
			GameTooltip:SetOwner(this, 'ANCHOR_BOTTOMRIGHT');
			GameTooltip:SetSpell(this.spellid, SpellBookFrame.bookType);
			if( this.max_rank ) then
				GameTooltip:AddLine('\nLeft Click '.. this.max_rank ..'\nRight Click ' .. this.mid_rank..')\nMiddle Click Rank(' .. this.min_rank);
			end
		end
	elseif ( BeneCastProfiles[profileid].ShowTooltips and BeneCastProfiles[profileid].ShowTooltipsName ) then
		GameTooltip:SetOwner(this, 'ANCHOR_BOTTOMRIGHT');
		if( this.max_rank ) then
			GameTooltip:SetText(this.spellname .. '\n\nLeft Click '.. this.max_rank ..'\nRight Click ' .. this.mid_rank..')\nMiddle Click Rank(' .. this.min_rank);
		else
			GameTooltip:SetText(this.spellname);
		end
	end
	if this.reason then
		if this.reason ~= '' then
			PutDebugMsg('Fade-reason: ' .. this.reason);
		end
	end

end

-- Cancel shadowform before casting a Holy spell
function BeneCastButton_PreClick()

	if BC_shadowformbuffid and BeneCastSetup.ShadowCancel then
		if this.spelltype == 'res' or
		   this.spelltype == 'efficient' or
		   this.spelltype == 'efficient2' or
		   this.spelltype == 'efficient3' or
		   this.spelltype == 'emergency' or
		   this.spelltype == 'emergency3' or
		   this.spelltype == 'instant' or
		   this.spelltype == 'group' or
		   this.spelltype == 'group2' or
		   this.spelltype == 'group3' or
		   this.spelltype == 'disease' or
		   this.spelltype == 'disease2' or
		   this.spelltype == 'selfbuff3' then
			CancelPlayerBuff(BC_shadowformbuffid);
		end
	end

end

-- Keep track of what spell will be cast, used for notification purposes
function BeneCastButton_PostClick(button)

	if this.notifyspell and button then
		local buttonid = tonumber(SecureButton_GetButtonSuffix(button));
		if this.notifyspell[buttonid] then
			BC_spellbeingcast = this.notifyspell[buttonid];
		end
	end

end

-- Update the cooldown for a button on events
function BeneCastButton_UpdateCooldown()

	-- If the unit for this button does not exist return
	if not UnitExists(this.unit) then
		return;
	end
	
	-- No need to update for a hidden button
	if ( not this:IsVisible() ) then
		return;
	end
	
	local spell;
	-- Figure out this button's spelltype
	local spelltype = this.spelltype;
	-- If there is no spell associated with the button
	if not spelltype or this.spellid == nil or this.spellid == 0 then
		return;
	end
	
	-- Cooldown stuff that I'm not quite sure how exactly it all works
	local cooldown = getglobal(this:GetName()..'Cooldown');
	local iconTexture = getglobal(this:GetName()..'Icon');
	local start, duration, enable = GetSpellCooldown(this.spellid, SpellBookFrame.bookType);
	CooldownFrame_SetTimer(cooldown, start, duration, enable);
	if ( enable == 1 ) then
		iconTexture:SetVertexColor(1.0, 1.0, 1.0);
	else
		iconTexture:SetVertexColor(0.4, 0.4, 0.4);
	end

end

-- Register events for BeneCastButtons
function BeneCastButton_OnLoad()

	this:RegisterEvent('SPELL_UPDATE_COOLDOWN');

end

-- *****************************************************************************
-- BeneCast Button Drawing/Updating Functions
-- *****************************************************************************

-- Function used to show a BeneCast button
-- Parameters spell_type, and the first free button available
-- 'Free' buttons are buttons that are not being used
-- Returns true if a button was created
function BeneCast_ShowButton(type, freebutton, id, member, ranknumber)

	if InCombatLockdown() then
		return;
	end
	local profileid = BeneCast_ProfileForPanelID(id);
	local made_button = nil;
	-- Figure out the class of member
	local class;
	if member then
		class = UnitClass(member);		
	end
	if not class then
		class = BENECAST_STRINGS.CLASS_WARRIOR;
	end
	local classnum = BC_classes[class];
	if classnum == nil then
		--PutMsg('Cant find classnum for class ' .. class);
		classnum = 8; 
	end
	
	--Only if a hunter/warlock is targetting his pet show the petheal
	if ( BC_class == BENECAST_STRINGS.CLASS_WARLOCK or BC_class == BENECAST_STRINGS.CLASS_HUNTER ) then
		if ( ( member ~= 'player' and not ( member == 'target' and UnitIsUnit('pet','target') ) ) and type == 'efficient' ) then
			return nil;
		end
	end
	
	-- If the player has the spell, wants it shown, and freebutton is <= BeneCastProfiles.ButtonNumber then continue
	local showbutton = nil;
	if ( member == 'target' or member == 'targettarget' ) then
		for k, v in pairs(BC_classes) do
			if BeneCastProfiles[profileid][v][type] then
				showbutton = true;
				do break end;
			end
		end
	elseif ( member == 'player' and BeneCastProfiles[profileid][1][type] ) then
		showbutton = true;
	elseif ( member ~= 'player' and BeneCastProfiles[profileid][classnum][type] ) then
		showbutton = true;
	end
	if ( BC_spell_data[type] and showbutton and freebutton <= BeneCastProfiles[profileid].ButtonNumber ) then
		local button = getglobal('BeneCastPanel' .. id .. 'Button' .. freebutton);
		-- Set made_button to true
		made_button = true;
		local icon = getglobal('BeneCastPanel' .. id .. 'Button' .. freebutton .. 'Icon');
		button.spelltype = type;
		if ranknumber == 0 then
			ranknumber = nil;
		end
		button.ranknumber = ranknumber;
		button.max_rank = nil;
		button.mid_rank = nil;
		button.min_rank = nil;
		if type == 'assist' then
			button:SetAttribute('type','assist');
			button:SetAttribute('unit',member);
			button.spellid = nil;
			button.spellname = BENECAST_STRINGS.TEXT_ASSIST;
			button.notifyspell = {};
			-- Set up the texture for the button
			icon:SetTexture('Interface\\Icons\\Ability_Marksmanship');
		elseif type == 'firstaid' then
			if BeneCastSetup.firstaiditem then
				button:SetAttribute('type','macro');
				button:SetAttribute('macrotext','/use [target=' .. member .. '] ' .. BeneCastSetup.firstaiditem);
			else
				button:SetAttribute('type','target');
				button:SetAttribute('unit',member);
			end
			local i = BC_spell_data[type].spellcount;
			button.spellid = BC_spell_data[type]['spells'][i].id;
			button.spellname = BENECAST_STRINGS.FIRST_AID;
			button.ranknumber = i;
			button.notifyspell = {};
			table.insert(button.notifyspell,BC_spell_data[type]['spells'][i]);
			table.insert(button.notifyspell,BC_spell_data[type]['spells'][i]);
			table.insert(button.notifyspell,BC_spell_data[type]['spells'][i]);
			-- Set up the texture for the button
			icon:SetTexture(BC_spell_data[type].texture);
		
		else
			button:SetAttribute('type',nil);
			button:RegisterForClicks("AnyUp");
			button:SetAttribute("*unit*", member);
			if not ranknumber then
				ranknumber = BC_spell_data[type].spellcount;
				button.ranknumber = ranknumber;
			end
			local spellname = BC_spell_data[type]['spells'][ranknumber].name;
			local spellnamemax, spellnamemid, spellnamemin;
			local hasrank = false;
			if BC_spell_data[type]['spells'][ranknumber].rank then
				spellnamemax = spellname .. '(' .. BC_spell_data[type]['spells'][ranknumber].rank .. ')';
				hasrank = true;
			else
				spellnamemax = spellname .. '()';
			end
			local ranknumbermid = math.ceil(ranknumber/2);
			if BC_spell_data[type]['spells'][ranknumbermid].rank then
				spellnamemid = spellname .. '(' .. BC_spell_data[type]['spells'][ranknumbermid].rank .. ')';
			else
				spellnamemid = spellname .. '()';
			end
			if BC_spell_data[type]['spells'][1].rank then
				spellnamemin = spellname .. '(' .. BC_spell_data[type]['spells'][1].rank .. ')';
			else
				spellnamemin = spellname .. '()';
			end
			
			button:SetAttribute("*helpbutton1", "heal1");
			button:SetAttribute("*type-heal1", "spell");
			button:SetAttribute("spell-heal1", spellnamemax); -- max rank	
			if hasrank then
				button.max_rank = ranknumber;
			end
			
			button:SetAttribute("*helpbutton2", "heal2");
			button:SetAttribute("*type-heal2", "spell");
			button:SetAttribute("spell-heal2", spellnamemid); -- mid rank
			if hasrank then
				button.mid_rank = ranknumbermid;
			end

			button:SetAttribute("*helpbutton3", "heal3");
			button:SetAttribute("*type-heal3", "spell");
			button:SetAttribute("spell-heal3", spellnamemin); -- min rank
			if hasrank then
				button.min_rank = 1;
			end

			button.spellid = BC_spell_data[type]['spells'][ranknumber].id;
			button.spellname = spellnamemax;
			button.notifyspell = {};
			table.insert(button.notifyspell,BC_spell_data[type]['spells'][ranknumber]);
			table.insert(button.notifyspell,BC_spell_data[type]['spells'][ranknumbermid]);
			table.insert(button.notifyspell,BC_spell_data[type]['spells'][1]);
			-- Set up the texture for the button
			icon:SetTexture(BC_spell_data[type].texture);	
		end
		button.unit = member;
		button:Show();
		button:SetAlpha(1); --reset alpha
		if member then
			if not BC_buttons_by_spell[member] then
				BC_buttons_by_spell[member] = {};
			end
			BC_buttons_by_spell[member][type] = button:GetName();
		end
	end

	return made_button;
	
end

-- Function to fade buttons if they're not usable
function BeneCast_SetButtonFade(buttonname, showbutton, reason)
	if buttonname == nil then
		return;
	end
	local button = getglobal(buttonname);
	if button == nil then
		return;
	end
	button.reason = reason;
	local fadeicon = getglobal(buttonname .. 'Fade');
	if showbutton then
		fadeicon:Hide();
	else
		fadeicon:Show();
	end
end

-- Function to hide temporary buttons if they're not to be shown
function BeneCast_HideButton(buttonnum, id, member)

	if InCombatLockdown() then
		return;
	end

	local button = getglobal('BeneCastPanel' .. id .. 'Button' .. buttonnum);
	local fadeicon = getglobal('BeneCastPanel' .. id .. 'Button' .. buttonnum .. 'Fade');
	if ( fadeicon:IsShown() ) then
		fadeicon:Hide();
	end
	local rangeicon = getglobal('BeneCastPanel' .. id .. 'Button' .. buttonnum .. 'Range');
	if ( rangeicon:IsShown() ) then
		rangeicon:Hide();
	end
	if ( button:IsShown() ) then
		button:Hide();
	end
	if button.spelltype then
		if BC_buttons_by_spell[member] then
			BC_buttons_by_spell[member][button.spelltype] = nil;
		end
	end

end

--WINTROW.6 START
-- Function used to update buttons for a unitid
function BeneCast_UpdateButtonsForUnit(unitid)
	local panelids;
	panelids = BeneCast_UnitID(unitid);
	if panelids then
		if type(panelids) == 'table' then
			for i, id in pairs (panelids) do
				BeneCast_UpdateButtons(id);
			end
		else
			BeneCast_UpdateButtons(panelids);
		end
	end
end
--WINTROW.6 STOP

-- Function used to update buttons for all possible targets
function BeneCast_UpdateButtons(id)

	-- Sometimes the id is nil
	if not id then
		PutDebugMsg('BeneCast_UpdateButtons(): parameter id is nil');
		return;
	end
	
	local profileid = BeneCast_ProfileForPanelID(id);	
	if not BC_attached_targets[id] then
		if id == 1 or id == 91 or id == 92 then
			PutDebugMsg('Not updating panel '..id..' because it was not previously attached.');
		end
		return;
	end
	BC_attached_targets[id] = false;
	
	if getglobal('BeneCastPanel' .. id) == nil then
		PutDebugMsg('Panel BeneCastPanel' .. id .. ' not found');
		return;
	end
	
	local panel = getglobal('BeneCastPanel' .. id);
	if not InCombatLockdown() then
		panel:Show();
	end
	
	-- Unit statusses
	local disableall = false;
	local unitexists = true;
	local unitisfriend = true;
	local hidepanel = false;

	if ( id == 92 and not BeneCastProfiles[profileid].ShowTargetTarget ) then
		disableall = true;
		hidepanel = true;
	end
	
	-- Figure out who this frame belongs too
	local member = BeneCast_GetUnitIDFromPanelID(id);
	if member == nil then
		unitexists = false;
		disableall = true;
		if not InCombatLockdown() then
			panel:Hide();
		end
		PutDebugMsg('No member found for panel ' .. id);
		return;
	else
		-- if this member is non existant the parent frame will hide the panel for us
		if not UnitExists(member) then
			unitexists = false;
			disableall = true;
		end
	end
		
	-- Hide this BeneCastPanel frame and return if this member is a raidid that is not being shown
	local name = UnitName(member);
	local raidenabled;
	if unitexists then
		if BeneCast_UnitIsRaidID(member) then
			raidenabled = BENECAST_RAID_LIST[name];
			if ( not raidenabled or not BeneCastProfiles[profileid].ShowRaid ) then
				disableall = true;
				hidepanel = true;
			end
		end
	end
	
	-- Hide this BeneCastPanel frame and return if this member is a pet and ShowPets is not enabled
	-- Except if the player is directly targetting the pet
	if ( member ~= 'target' and member ~= 'targettarget' ) and unitexists and BeneCast_UnitIsPetID(member) then
		if not BeneCastProfiles[profileid].ShowPets then
			disableall = true;
			hidepanel = true;
		elseif BeneCast_UnitIsRaidPetID(member) then
			if not BeneCastProfiles[profileid].ShowRaidPets then
				disableall = true;
				hidepanel = true;
			else
				local owner = BeneCast_GetRaidPetOwnerID(member);
				local ownername = UnitName(owner);
				raidenabled = BENECAST_RAID_LIST[ownername];
				if not raidenabled then
					disableall = true;
					hidepanel = true;
				end
			end
		end
	end
	
	-- This is a hide due to settings, hide and exit if possible
	if hidepanel then
		if not InCombatLockdown() then
			panel:Hide();
			return;
		end
	end
	
	-- Remember if member is mindcontrolled
	local curemagic = false;
	if unitexists then
		if UnitIsCharmed(member) then
			curemagic = true;
		end
	end
	
	-- Disable this BeneCastPanel frame if member is not friendly
	if unitexists and member ~= 'player' and member ~= 'pet' then
		if ( not UnitIsFriend('player', member) and not curemagic ) then
			disableall = true;
			unitisfriend = false;
		end
	end
	
	-- If the target is invalid due to a temporary game-status keep it hidden if not in combat
	if not unitisfriend or not unitexists or disableall then
		if not InCombatLockdown() and not BC_ConsiderInCombat then	
			panel:Hide();
			return;
		end
	end
	
	if unitexists then
		if ( UnitIsGhost('player') or not UnitIsConnected(member) ) then
			disableall = true;
		end
	end
	
	-- If the player can ressurect this member then show the res button and return
	-- 	If there is a buff showing (UnitBuff()) then the player _cannot_ be dead, or they
	-- 	are a priest in Spirit of Redemption form, and so they shouldn't be ressurected.
	-- 	DEAD = No buffs.  Hunters do not need a special case here.
	-- 	We don't bother with ghosts, because we can't cast it on their name, we have to find
	-- 	their body.
	local showres = false;
	if unitexists and unitisfriend then
		if ( UnitIsDead(member) and not UnitBuff(member, 1) ) then
			showres = true;
		end
	end
	
	local activeshapeshiftid = 0;
	local activeshapeshiftname = 'Caster';
	-- If player is a Druid and is shapeshifted hide all buttons
	if ( BC_class == BENECAST_STRINGS.CLASS_DRUID ) then
		-- Check all shapeshift forms to see if they're active
		local shapeshiftnum = GetNumShapeshiftForms();
		local icon, name, active;
		for i = 1,shapeshiftnum do
			icon, name, active = GetShapeshiftFormInfo(i);
			if ( active ) then
				activeshapeshiftid = i;
				activeshapeshiftname = name;
				do break end
			end
		end
	end

	-- Otherwise show the buttons. Show the BeneCastPanel frame first
	if not InCombatLockdown() then
		panel:Show();
	end
	
	-- If there is no class, set it to warrior
	local class;
	if unitexists then
		class = UnitClass(member);
	end
	if not class then
		class = BENECAST_STRINGS.CLASS_WARRIOR;
	end
	
	local buttonstochange = {};
	for i, spelltype in pairs(BC_spellsort) do
		buttonstochange[spelltype] = {};
	end
	buttonstochange['assist'] = {};

	-- Show assist button
	buttonstochange['assist'].add = true;
	buttonstochange['assist'].show = true;
	
	-- Show Resurrection button
	buttonstochange['res'].add = true;
	buttonstochange['res'].show = showres;
	if not showres then
		buttonstochange['res'].reason = 'Unit not ressurectable';
	end
	
	-- Show First Aid
	buttonstochange['firstaid'].add = true;
	buttonstochange['firstaid'].show = (BeneCastSetup.firstaiditem ~= nil) and (activeshapeshiftid == 0);
	
	-- Show Heal buttons as persistant buttons
	buttonstochange['efficient'].add = true;
	buttonstochange['efficient2'].add = true;
	buttonstochange['efficient3'].add = true;
	buttonstochange['emergency'].add = true;
	buttonstochange['emergency2'].add = true;
	buttonstochange['emergency3'].add = true;
	buttonstochange['emergency4'].add = true;
	buttonstochange['instant'].add = true;
	buttonstochange['instant3'].add = true;
	
	buttonstochange['efficient'].show = (activeshapeshiftid == 0);
	buttonstochange['efficient2'].show = true;
	buttonstochange['efficient3'].show = true;
	buttonstochange['emergency4'].show = true;
	buttonstochange['emergency'].show = (activeshapeshiftid == 0) or (activeshapeshiftname == BENECAST_STRINGS.TREE_OF_LIFE);
	buttonstochange['emergency2'].show = true;
	buttonstochange['instant'].show = (activeshapeshiftid == 0) or (activeshapeshiftname == BENECAST_STRINGS.TREE_OF_LIFE);
	buttonstochange['instant3'].show = (activeshapeshiftid == 0) or (activeshapeshiftname == BENECAST_STRINGS.TREE_OF_LIFE);
	if (activeshapeshiftid ~= 0) then
		buttonstochange['efficient'].reason = 'activeshapeshiftid = ' .. activeshapeshiftid;
		buttonstochange['efficient2'].reason = 'activeshapeshiftid = ' .. activeshapeshiftid;
		buttonstochange['efficient3'].reason = 'activeshapeshiftid = ' .. activeshapeshiftid;
	end
	if (activeshapeshiftid ~= 0) and (activeshapeshiftname ~= BENECAST_STRINGS.TREE_OF_LIFE) then
		buttonstochange['emergency'].reason = 'activeshapeshiftid = ' .. activeshapeshiftid;
		buttonstochange['instant'].reason = 'activeshapeshiftid = ' .. activeshapeshiftid;
	end
	if (activeshapeshiftname ~= BENECAST_STRINGS.TREE_OF_LIFE) then
		buttonstochange['instant3'].reason = 'activeshapeshiftname ~= ' .. BENECAST_STRINGS.TREE_OF_LIFE;
	end
	
	-- Paladin's Lay on Hands spell
	buttonstochange['loh'].add = true;
	buttonstochange['loh'].show = true;

	-- Group heals are handled differently than other heals
	-- Only on the Shaman Group heal does targeting matter
	-- Show up only on player unless class is Shaman
	buttonstochange['group'].add = 
		( BC_class == BENECAST_STRINGS.CLASS_SHAMAN or member == 'player' );
	buttonstochange['group2'].add = ( member == 'player' );
	buttonstochange['group3'].add = true;
	buttonstochange['group'].show = (activeshapeshiftid == 0) or (activeshapeshiftname == BENECAST_STRINGS.TREE_OF_LIFE);
	buttonstochange['group2'].show = true; -- Druids don't have this spell
	buttonstochange['group3'].show = true; -- Druids don't have this spell
	if (activeshapeshiftid ~= 0) and (activeshapeshiftname ~= BENECAST_STRINGS.TREE_OF_LIFE) then
		buttonstochange['group'].reason = 'activeshapeshiftid = ' .. activeshapeshiftid;
	end
	
	-- Check to see if the member can be cured of a harmful effect
	-- Do this before showing buffs so Weakened Soul is detected before drawing buffs
	-- Iterator for debuffs
	local i = 1;
	
	-- Variables to determine if cures should be shown
	-- This is done in case there are multiples of one kind of effect
	local poisonshow = nil;
	local diseaseshow = nil;
	local magicshow = nil;
	local curseshow = nil;
	
	local weakenedsoul = nil;
	BC_spirit_of_redemption = nil;
	
	-- Cycle through the Units Debuffs to find removable effects
	if not BeneCast_Tooltip:IsOwned(WorldFrame) then
		BeneCast_Tooltip:SetOwner(WorldFrame, 'ANCHOR_NONE');
	end

	if unitexists and unitisfriend then
		local debuffname, dispeltype;	
		while ( UnitDebuff(member, i) ) do

			debuffname, _, _, _, dispeltype = UnitDebuff(member, i);
			if ( debuffname ) then
				-- Check to see if this debuff is Weakened Soul and save it for when you get to the buffs
				if ( debuffname == BENECAST_STRINGS.AILMENT_WEAKENED_SOUL ) then
					weakenedsoul = true;
				end
				if ( debuffname == BENECAST_STRINGS.AILMENT_SPIRIT_OF_REDEMPTION ) then
					BC_spirit_of_redemption = true;
				end
	
				if dispeltype == 'Poison' then
					poisonshow = true;
				elseif ( dispeltype == 'Disease' ) then
					-- Paladins can cure diseases but don't have a spell with the 'disease' spelltype
					-- The Paladin spell that cures poisons also cures diseases
					if ( BC_class == BENECAST_STRINGS.CLASS_PALADIN ) then
						poisonshow = true;
					else
						diseaseshow = true;
					end
				elseif ( dispeltype == 'Magic' ) then
					-- Paladins can cure magic with Cleanse which is of the 'poison' spelltype
					-- If the Paladin has Cleanse have it show Cleanse
					if ( BC_class == BENECAST_STRINGS.CLASS_PALADIN and BC_spell_data['poison'] ) then
						if BC_spell_data['poison']['spells'][2] then
							poisonshow = true;
						else
							magicshow = true;
						end
					else
						magicshow = true;
					end
				elseif ( dispeltype == 'Curse' ) then
					curseshow = true;
				end
			end
			i = i + 1;
		end
	end
	
	-- Show Buff buttons as temporary buttons
	
	-- See if the groupbuff is on the intended target
	local groupbuffsshown = {};
	local groupblessingsshown = {};
	
	-- See if a mage's ward is on the player
	local wardshown = nil;
			
	-- Check to see if this member does not have castable buffs
	
	-- Reset the iterator for buffs
	i = 1;
	
	-- Clear all entries in BC_buffs_in_effect
	for j in pairs (BC_buffs_in_effect) do
		BC_buffs_in_effect[j].found = false;
	end
	
	if unitexists and unitisfriend then
		if UnitIsUnit(member,'player') then
			-- Set clearcasting state to nil
			BC_clearcasting = nil;
			local old_innerfocus = BC_innerfocus; 
			BC_innerfocus = nil; 
			BC_shadowformbuffid = nil;
		end
	
		-- Cycle through the Units Buffs to find if the effect is on the unit
		local buffname = nil;
		while ( UnitBuff(member, i) ) do
				
			buffname = UnitBuff(member, i);
			if ( buffname ) then
				-- Only check these buffs if we're checking out the player ! (Thnx Shimavak)
				if UnitIsUnit(member,'player') then
					-- If the buff is Clearcasting then set clearcasting state to true 
					if ( buffname == BENECAST_STRINGS.BUFF_CLEARCASTING ) then
						BC_clearcasting = true;
					end
					if ( buffname == BENECAST_STRINGS.SELFBUFF_INNER_FOCUS ) then
						BC_innerfocus = true;
					end
					if ( buffname == BENECAST_STRINGS.AILMENT_SPIRIT_OF_REDEMPTION ) then
						BC_spirit_of_redemption = true;
					end
				end
				-- Now, if the buff is one of the uninteractable ones, we'll turn off the whole set of buttons				
				if ( buffname == BENECAST_STRINGS.BUFF_PHASE_SHIFT or buffname == BENECAST_STRINGS.PARTYBUFF_DIVINE_INTERVENTION ) then
					disableall = true;
				end
			
				local effect = BeneCast_SpellTypes[BC_class][buffname];
				-- If the effect is a groupbuff set groupbuffshown to true
				if ( effect ) then
					if ( ( BC_class == BENECAST_STRINGS.CLASS_PRIEST or 
					       BC_class == BENECAST_STRINGS.CLASS_DRUID or 
					       BC_class == BENECAST_STRINGS.CLASS_MAGE ) and string.find(effect,'groupbuff(.+)') ) then
					        local tmp;
					        _, _, tmp = string.find(effect,'groupbuff(.+)');
						table.insert(groupbuffsshown,tmp);
					elseif ( BC_class == BENECAST_STRINGS.CLASS_PALADIN and string.find(effect,'buffparty(.+)g') ) then
						local tmp;
						_, _, tmp = string.find(effect,'buffparty(.+)g');
						table.insert(groupblessingsshown,tmp);
					-- If the effect is a mage ward set wardshown to true
					elseif ( BC_class == BENECAST_STRINGS.CLASS_MAGE and ( effect == 'selfbuff12' or effect == 'selfbuff13' ) ) then
						wardshown = true;
					end 
					-- If the effect is shadowform
					if member == 'player' and BC_class == BENECAST_STRINGS.CLASS_PRIEST and effect == 'selfbuff10' then
						BC_shadowformbuffid = i;
					end

					if ( BC_buffs_in_effect_by_name[effect] ) then
						local j = BC_buffs_in_effect_by_name[effect];
						-- Set found for the type to true
						BC_buffs_in_effect[j].found = true;
					else
						PutDebugMsg('Buff ' .. effect .. ' not found in BC_buffs_in_effect_by_name-table');
					end
				end
			else
				PutDebugMsg('buffname was false!');
			end
			i = i + 1;
		
		end
	end
	
	if BeneCastSetup.Debug then
		if BC_innerfocus and not old_innerfocus then
			PutDebugMsg('Detected Inner Focus');
		end
	end
	
	-- Make _buff_shown for PW:F, AI, or MotW on the intended target true if they have the groupbuff on them
	if ( ( BC_class == BENECAST_STRINGS.CLASS_PRIEST or 
	       BC_class == BENECAST_STRINGS.CLASS_DRUID or 
	       BC_class == BENECAST_STRINGS.CLASS_MAGE ) and table.getn(groupbuffsshown) > 0 ) then 
		for k, buffno in pairs (groupbuffsshown) do
			local effect = 'buff' .. buffno;
			local j = nil;
			if ( BC_buffs_in_effect_by_name[effect] ) then
				j = BC_buffs_in_effect_by_name[effect];
			end
			if j ~= nil then
				BC_buffs_in_effect[j].found = true;
			end
		end
	elseif ( BC_class == BENECAST_STRINGS.CLASS_PALADIN and table.getn(groupblessingsshown) > 0 ) then
		for k, blessingno in pairs (groupblessingsshown) do 
			local effect = 'buff' .. blessingno;
			local j = nil;
			if ( BC_buffs_in_effect_by_name[effect] ) then
				j = BC_buffs_in_effect_by_name[effect];
			else
				effect = 'partybuff' .. blessingno;
				if ( BC_buffs_in_effect_by_name[effect] ) then
					j = BC_buffs_in_effect_by_name[effect];
				end
			end
			if j ~= nil then
				BC_buffs_in_effect[j].found = true;
			end
		end  
	elseif ( BC_class == BENECAST_STRINGS.CLASS_MAGE and wardshown ) then
		if ( BC_buffs_in_effect_by_name['selfbuff12'] ) then
			local j = BC_buffs_in_effect_by_name['selfbuff12'];
			BC_buffs_in_effect[j].found = true;
		end
		if ( BC_buffs_in_effect_by_name['selfbuff13'] ) then
			local j = BC_buffs_in_effect_by_name['selfbuff13'];
			BC_buffs_in_effect[j].found = true;
		end
	end
	
	--Show Swiftmend if applicable
	buttonstochange['instant2'].add = true;
	if unitisfriend then
		if ( BC_class == BENECAST_STRINGS.CLASS_DRUID and (activeshapeshiftid == 0 or activeshapeshiftname == BENECAST_STRINGS.TREE_OF_LIFE) ) then
			buttonstochange['instant2'].show = 
				( BC_buffs_in_effect[46].found or BC_buffs_in_effect[47].found );
			if not buttonstochange['instant2'].show then
				buttonstochange['instant2'].reason = 'No HoT active';
			end
		else
			buttonstochange['instant2'].show = false;
			if BC_class ~= BENECAST_STRINGS.CLASS_DRUID then
				buttonstochange['instant2'].reason = 'Not a Druid';
			else
				buttonstochange['instant2'].reason = 'activeshapeshiftid = ' .. activeshapeshiftid;
			end
		end
	end
	
	-- Show buff buttons of buffs not in effect if possible
	local showselfbuff, showpartybuff;
	if member then
		showselfbuff = ( member == 'player' );
		showpartybuff = ( member == 'player' or 
			  	UnitInParty(member) or 
			        BeneCast_UnitInRaid(member) or BeneCast_UnitIsPartyPet(member) or 
				BeneCast_UnitIsRaidPet(member) );
	else
		showselfbuff = true;
		showpartybuff = true;
	end
	
	local reason;	
	for j in pairs (BC_buffs_in_effect) do
		-- Some buttons should not show on others
		-- Selfbuffs on others and buffparty on those outside of the party/raid
		-- Groupbuffs such as Prayer of Fortitude should only be shown on the target/player
		if not BC_buffs_in_effect[j].notabuff then
			reason = nil;
			if string.find(BC_buffs_in_effect[j].name, 'party') and not showpartybuff and
			   ( member == 'target' and BC_ConsiderInCombat ) then
				BC_buffs_in_effect[j].found = true;
				reason = 'Not in party';
			end
			if ( string.find(BC_buffs_in_effect[j].name, 'groupbuff') and not showselfbuff ) then
				-- Do nothing
			elseif ( string.find(BC_buffs_in_effect[j].name, 'self') and not showselfbuff ) then
				-- Do nothing
			elseif string.find(BC_buffs_in_effect[j].name, 'party') and 
			       not showpartybuff and not ( member == 'target' and BC_ConsiderInCombat ) then
				-- Do nothing
			else
				-- First show the button
				buttonstochange[BC_buffs_in_effect[j].name].add = true;
				local allowedinthisform = true;
				if BC_class == BENECAST_STRINGS.CLASS_DRUID and activeshapeshiftid ~= 0 then
					allowedinthisform = false;
					if activeshapeshiftname == BENECAST_STRINGS.MOONKIN then
						if ( BC_buffs_in_effect[j].name == 'selfbuff2' ) then
							allowedinthisform = true;
						end
						if ( BC_buffs_in_effect[j].name == 'selfbuff3' ) then
							allowedinthisform = true;
						end
						if ( BC_buffs_in_effect[j].name == 'selfbuff4' ) then
							allowedinthisform = true;
						end
						if ( BC_buffs_in_effect[j].name == 'buff2' ) then
							allowedinthisform = true;
						end
					end
					if activeshapeshiftname == BENECAST_STRINGS.TREE_OF_LIFE then
						if ( BC_buffs_in_effect[j].name == 'selfbuff1' ) then
							allowedinthisform = true;
						end
						if ( BC_buffs_in_effect[j].name == 'selfbuff4' ) then
							allowedinthisform = true;
						end
						if ( BC_buffs_in_effect[j].name == 'buff3' ) then
							allowedinthisform = true;
						end
					end
				end				
				if allowedinthisform then --WINTROW.2 & 4
					-- Then see about fading it
					local found = BC_buffs_in_effect[j].found;
					if not found then
						-- Make this button fade if it is for PW:S and the effect Weakened Soul is on the target
						if ( BC_class == BENECAST_STRINGS.CLASS_PRIEST and BC_buffs_in_effect[j].name == 'partybuff1' and weakenedsoul ) then
							found = true;
							buttonstochange[BC_buffs_in_effect[j].name].reason = 'Weakened Soul is active';
						end
					else
						if reason then
							buttonstochange[BC_buffs_in_effect[j].name].reason = reason;
						else
							buttonstochange[BC_buffs_in_effect[j].name].reason = 'Buff is active';
						end
					end
					buttonstochange[BC_buffs_in_effect[j].name].show = not found;
				else
					buttonstochange[BC_buffs_in_effect[j].name].show = false;
					buttonstochange[BC_buffs_in_effect[j].name].reason = 'activeshapeshiftid = ' .. activeshapeshiftid;
				end 
			end
		else
			--if BC_buffs_in_effect[j].name == 'instant' then
			--	PutDebugMsg('Skipping "instant" as a buff-button');
			--end
		end
	end
	
	-- Show Weapon Enchants for Shaman and Priests
	if ( member == 'player') then
		if ( BC_class == BENECAST_STRINGS.CLASS_SHAMAN ) then
			-- Show all weaponenchants
			buttonstochange['weaponenchant1'].add = true;
			buttonstochange['weaponenchant2'].add = true;
			buttonstochange['weaponenchant3'].add = true;
			buttonstochange['weaponenchant4'].add = true;
			-- See about fading them
			buttonstochange['weaponenchant1'].show = 
				( BC_weaponenchant ~= BENECAST_STRINGS.WEAPONENCHANT_ROCKBITER );
			if ( BC_weaponenchant == BENECAST_STRINGS.WEAPONENCHANT_ROCKBITER ) then
				buttonstochange['weaponenchant1'].reason = 'Buff is active';
			end
			buttonstochange['weaponenchant2'].show = 
				( BC_weaponenchant ~= BENECAST_STRINGS.WEAPONENCHANT_FLAMETONGUE );
			if ( BC_weaponenchant == BENECAST_STRINGS.WEAPONENCHANT_FLAMETONGUE ) then
				buttonstochange['weaponenchant2'].reason = 'Buff is active';
			end
			buttonstochange['weaponenchant3'].show = 
				( BC_weaponenchant ~= BENECAST_STRINGS.WEAPONENCHANT_FROSTBRAND );
			if ( BC_weaponenchant == BENECAST_STRINGS.WEAPONENCHANT_FROSTBRAND ) then
				buttonstochange['weaponenchant3'].reason = 'Buff is active';
			end
			buttonstochange['weaponenchant4'].show = 
				( BC_weaponenchant ~= BENECAST_STRINGS.WEAPONENCHANT_WINDFURY );
			if ( BC_weaponenchant == BENECAST_STRINGS.WEAPONENCHANT_WINDFURY ) then
				buttonstochange['weaponenchant4'].reason = 'Buff is active';
			end
		end
		if ( BC_class == BENECAST_STRINGS.CLASS_PRIEST ) then
			buttonstochange['weaponenchant1'].add = true;
			buttonstochange['weaponenchant1'].show = 			
				( BC_weaponenchant ~= BENECAST_STRINGS.WEAPONENCHANT_FEEDBACK );
			if ( BC_weaponenchant == BENECAST_STRINGS.WEAPONENCHANT_FEEDBACK ) then
				buttonstochange['weaponenchant1'].reason = 'Buff is active';
			end
		end
	end
	
	-- Show Cure buttons as temporary buttons
	
	if ( BC_buffs_in_effect[48].fade ) then
		poisonshow = true; --for flashing the poison button while Abolish Poison is active
	end
	
	if curemagic then
		magicshow = true;
	end
		
	-- Show buttons for effects that are removable once
	-- First show the buttons
	buttonstochange['poison'].add = true;
	buttonstochange['disease'].add = true;
	buttonstochange['disease2'].add = true;
	buttonstochange['magic'].add = true;
	buttonstochange['curse'].add = true;
	if unitisfriend then
		buttonstochange['poison'].show = (poisonshow and activeshapeshiftid == 0);
		if activeshapeshiftid ~= 0 then
			buttonstochange['poison'].reason = 'activeshapeshiftid = ' .. activeshapeshiftid;
		elseif not poisonshow then
			buttonstochange['poison'].reason = 'No Dispellable debuff found';
		end
		buttonstochange['disease'].show = diseaseshow;
		if not diseaseshow then
			buttonstochange['disease'].reason = 'No Dispellable debuff found';
		end
		buttonstochange['disease2'].show = diseaseshow;
		if not diseaseshow then
			buttonstochange['disease2'].reason = 'No Dispellable debuff found';
		end
		buttonstochange['magic'].show = magicshow;
		if not diseaseshow then
			buttonstochange['magic'].reason = 'No Dispellable debuff found';
		end
		buttonstochange['curse'].show = (curseshow and activeshapeshiftid == 0);
		if activeshapeshiftid ~= 0 then
			buttonstochange['curse'].reason = 'activeshapeshiftid = ' .. activeshapeshiftid;
		elseif not poisonshow then
			buttonstochange['curse'].reason = 'No Dispellable debuff found';
		end
	end
	
	if disableall then
		for spelltype in pairs(buttonstochange) do
			if spelltype == 'res' and showres then
				-- Do Nothing
			elseif spelltype == 'magic' and curemagic then
				-- Do Nothing
			else
				buttonstochange[spelltype].show = false;
				buttonstochange[spelltype].reason = 'Not applicable';
			end
		end
	end
	
	local showallbuttons;
	if InCombatLockdown() or BC_ConsiderInCombat then
		showallbuttons = true
	else
		showallbuttons = BeneCastProfiles[profileid].ShowAllButtons;
	end
	
	if not showallbuttons then
		for spelltype in pairs(buttonstochange) do
			if not buttonstochange[spelltype].show then
				buttonstochange[spelltype].add = false;
			end
		end
	end
	
	local firstfreebutton = 1;
	if not InCombatLockdown() then
		-- Hide all buttons before redrawing them
		for x = firstfreebutton, BENECAST_MAX_NUMBER_OF_BUTTONS do
			BeneCast_HideButton(x, id, member);
		end
	end
	for i, spelltype in pairs(BC_spellsort) do
		if not InCombatLockdown() then
			if buttonstochange[spelltype].add then
				if ( BeneCast_ShowButton(spelltype, firstfreebutton, id, member) ) then
					firstfreebutton = firstfreebutton+1;
				end
			end
		end
		if BC_buttons_by_spell[member] then
			if disableall or not buttonstochange[spelltype].show then
				BeneCast_SetButtonFade(BC_buttons_by_spell[member][spelltype],false,buttonstochange[spelltype].reason);
			else
				BeneCast_SetButtonFade(BC_buttons_by_spell[member][spelltype],true,'');
			end
		end
	end
	
	--Check for flashing
	local iterator = 1; -- First clean out the flash-buttons of this panel, ready to be re-added if still active
	while iterator <= table.getn(BC_buttons_that_flash) do
		if string.find(BC_buttons_that_flash[iterator],'BeneCastPanel' .. id .. 'Button') then
			table.remove(BC_buttons_that_flash,iterator);
		else
			iterator = iterator + 1;
		end
	end
	if BC_buttons_by_spell[member] then
		if ( BC_class == BENECAST_STRINGS.CLASS_PRIEST ) then
			if BC_buffs_in_effect[46].fade then
				PutDebugMsg('"instant" is in effect');
				if BC_buttons_by_spell[member]['instant'] then
					if BeneCastProfiles[profileid].FlashAsFade then
						getglobal(BC_buttons_by_spell[member]['instant'] .. 'Fade'):Show();
					else
						table.insert(BC_buttons_that_flash,BC_buttons_by_spell[member]['instant']);
					end
				else
					PutDebugMsg('BC_buttons_by_spell[' .. member .. '][instant] is nil');
				end
			end
		end
		if ( BC_class == BENECAST_STRINGS.CLASS_DRUID ) then
			if BC_buffs_in_effect[46].fade then
				PutDebugMsg('"instant" is in effect');				
				if BC_buttons_by_spell[member]['instant'] then
					if BeneCastProfiles[profileid].FlashAsFade then
						getglobal(BC_buttons_by_spell[member]['instant'] .. 'Fade'):Show();
					else
						table.insert(BC_buttons_that_flash,BC_buttons_by_spell[member]['instant']);
					end
				else
					PutDebugMsg('BC_buttons_by_spell[' .. member .. '][instant] is nil');
				end
			end
			if BC_buffs_in_effect[47].fade then
				PutDebugMsg('"emergency" is in effect');
				if BC_buttons_by_spell[member]['emergency'] then
					if BeneCastProfiles[profileid].FlashAsFade then
						getglobal(BC_buttons_by_spell[member]['emergency'] .. 'Fade'):Show();
					else
						table.insert(BC_buttons_that_flash,BC_buttons_by_spell[member]['emergency']);
					end
				else
					PutDebugMsg('BC_buttons_by_spell[' .. member .. '][emergency] is nil');
				end
			end
			if BC_buffs_in_effect[48].fade then
				PutDebugMsg('"poison" is in effect');
				if BC_buttons_by_spell[member]['poison'] then
					if BeneCastProfiles[profileid].FlashAsFade then
						getglobal(BC_buttons_by_spell[member]['poison'] .. 'Fade'):Show();
					else
						table.insert(BC_buttons_that_flash,BC_buttons_by_spell[member]['poison']);
					end
				else
					PutDebugMsg('BC_buttons_by_spell[' .. member .. '][poison] is nil');
				end
			end
		end
		if ( BC_class == BENECAST_STRINGS.CLASS_SHAMAN ) then
			if BC_buffs_in_effect[48].fade then
				PutDebugMsg('"poison" is in effect');
				if BC_buttons_by_spell[member]['poison'] then
					if BeneCastProfiles[profileid].FlashAsFade then
						getglobal(BC_buttons_by_spell[member]['poison'] .. 'Fade'):Show();
					else
						table.insert(BC_buttons_that_flash,BC_buttons_by_spell[member]['poison']);
					end
				else
					PutDebugMsg('BC_buttons_by_spell[' .. member .. '][poison] is nil');
				end
			end
		end
	end

	-- Hide all buttons that shouldn't be shown
	if not InCombatLockdown() then
		for x = firstfreebutton, BeneCastProfiles[profileid].ButtonNumber do
			BeneCast_HideButton(x, id, member);
		end
	end
	
	-- There is not reason that we would hide this later, so we can check this panel each
	-- 	time we have a reason to suspect a change.
	BC_attached_targets[id] = true;
	
	-- If there are no buttons to be shown, hide the frame
	if ( firstfreebutton == 1 and not InCombatLockdown() ) then
		panel:Hide();
		return;
	end
	
	-- Resize the BeneCastPanelFrame based on the number of buttons
	BeneCast_ResizeButtonsFor(id);
		
end


-- *****************************************************************************
-- BeneCastPanelMenu functions
-- *****************************************************************************

function BeneCastPanelDropDown_Initialize()
	
	local info;
	-- Add Title Button
	info = {};
	if ( UnitExists(BC_targets[this:GetID()]) ) then
		info.text = UnitName(BC_targets[this:GetID()]);
	else
		info.text = BENECAST_STRINGS.TEXT_UNKNOWN
	end
	info.isTitle = 1;
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
	
	-- Add Lock Button
	info = {};
	info.text = BENECAST_STRINGS.TEXT_LOCKED;
	info.value = this:GetID();
	info.checked = this.locked;
	info.func = BeneCastPanel_SetLock;
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
	
	-- Add Orientation Button
	info = {};
	info.text = 'Change Orientation';
	info.value = this:GetID() .. 'orient';
	info.func = BeneCastPanel_SetOrientation;
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

end

function BeneCastPanel_SetLock() 

	local frame = getglobal('BeneCastPanel' .. this.value);
	frame.locked = not frame.locked;
	
end

function BeneCastPanel_SetOrientation()

	local id = nil;
	_, _, id = string.find(this.value,'(.+)orient');
	id = tonumber(id);
	local profileid = BeneCast_ProfileForPanelID(id);
	PutDebugMsg('BeneCast_SetOrientation() called');
	if not BeneCastProfiles[profileid]['BeneCastPanel' .. id] then
		BeneCastProfiles[profileid]['BeneCastPanel' .. id] = {};
	end
	if BeneCastProfiles[profileid]['BeneCastPanel' .. id]['Orientation'] == 'down' then
		BeneCastProfiles[profileid]['BeneCastPanel' .. id]['Orientation'] = 'right';
	else
		BeneCastProfiles[profileid]['BeneCastPanel' .. id]['Orientation'] = 'down';
	end
	PutDebugMsg('New value BeneCastProfiles[' .. profileid .. '][BeneCastPanel' .. id .. ']["Orientation"] = ' ..
	            BeneCastProfiles[profileid]['BeneCastPanel' .. id].Orientation);
	BeneCast_OrientButtons(id);

end

function BeneCastPanelMenu_OnLoad()

	UIDropDownMenu_Initialize(this, BeneCastPanelDropDown_Initialize, 'MENU');

end

-- *****************************************************************************
-- BeneCastPanel functions
-- *****************************************************************************

function BeneCastPanel_OnLoad()

	-- set our default colors
	this:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b);
	this:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b);
	this.locked = 1;

end

function BeneCastPanel_OnMouseDown()

	local profileid = BeneCast_ProfileForPanelID(this:GetID());
	if ( ( ( not this.locked ) or ( this.locked == 0 ) or BeneCastProfiles[profileid].Unlock ) and ( arg1 == 'LeftButton' ) ) then
		this:StartMoving();
		this.isMoving = true;
	end
	
end

--WINTROW.6 START
--Moved this from the XML-code so I can add code to save the panel's locations
function BeneCastPanel_OnMouseUp()

	local id = this:GetID();
	local profileid = BeneCast_ProfileForPanelID(id);
	if ( this.isMoving ) then
		this:StopMovingOrSizing();
		this.isMoving = false;
		local framename = 'BeneCastPanel' .. id;
		if not BeneCastProfiles[profileid][framename] then
			BeneCastProfiles[profileid][framename] = {
				['Y'] = 0,
				['X'] = 0,
				['Point'] = 'TOPLEFT',
				['RelativePoint'] = 'TOPLEFT',
				['Frame'] = '',
				['Moved'] = true,
			};
		else
			BeneCastProfiles[profileid][framename]['Point'] = 'TOPLEFT';
			BeneCastProfiles[profileid][framename]['RelativePoint'] = 'TOPLEFT';
		end
		local parent = this:GetParent();
		-- The values stored are unscaled values (real pixels)
		-- When being applied they will need to be adjusted to the scale between the frame and his parent
		BeneCastProfiles[profileid][framename]['X'] = ( this:GetLeft() * this:GetEffectiveScale() ) - 
						 ( parent:GetLeft() * parent:GetEffectiveScale() ) + 0.5;
		BeneCastProfiles[profileid][framename]['X'] = math.floor(BeneCastProfiles[profileid][framename]['X']);
		BeneCastProfiles[profileid][framename]['Y'] = ( this:GetTop() * this:GetEffectiveScale() ) -
						 ( parent:GetTop() * parent:GetEffectiveScale() ) + 0.5;
		BeneCastProfiles[profileid][framename]['Y'] = math.floor(BeneCastProfiles[profileid][framename]['Y']);
		BeneCastProfiles[profileid].SnapTo = 'CUSTOM';
		BeneCastProfiles[profileid][framename]['Moved'] = true;
		UIDropDownMenu_SetSelectedValue(BeneCastSnapToMenu, BeneCastProfiles[profileid].SnapTo);
		-- Make sure BeneCast does all the moving around here... Thanks Snidely!
		if this:IsUserPlaced() then
			this:SetUserPlaced(false);
		end
		BeneCast_AttachPanelsFromVar(this:GetID());
		BeneCastUpdateCustomEditBoxes();
	end
	
end

function BeneCastPanel_OnEnter()
	if BeneCastSetup.Debug then
		local id, unitname, target, framename;
		id = this:GetID();

		framename = this:GetName();
		if not id then
			local idstr;
			_, _, idstr = string.find(framename,'BeneCastPanel(.+)');
			id = tonumber(idstr);
		end
		if id then
			target = BeneCast_GetUnitIDFromPanelID(id);
			if target then
				unitname = UnitName(target);
			end
		end
		if not target then
			target = 'nil';
		end
		if not unitname then
			unitname = 'nil';
		end
		if not id then
			id = '?';
		end
		PutDebugMsg('Mousing over ' .. framename .. ', targetid = ' .. target .. ', name is ' .. unitname);
	end
end
--WINTROW.6 STOP

-- *****************************************************************************
-- BeneCastOptionFrameTab functions
-- *****************************************************************************

function BeneCastOptionFrameTab_OnClick()

	-- Show the correct subframe based on which tab was clicked
	-- Hide all subframes that do not have matching ID numbers, show the matching subframe
	for subframe, subframename in pairs (BeneCastOptionFrameSubframes) do
		if this:GetID() == getglobal(subframename):GetID() then
			getglobal(subframename):Show();
		else
			getglobal(subframename):Hide();
		end
	end
	
	-- Play a sound for player feedback
	PlaySound('igCharacterInfoTab');
	
	-- Set the glowing checked status for the correct tab
	for x in pairs (BeneCastOptionFrameSubframes) do
		if ( x == this:GetID() ) then
			this:Disable()
			this:SetChecked(1);
		else
			getglobal('BeneCastOptionFrameTab' .. x):Enable();
			getglobal('BeneCastOptionFrameTab' .. x):SetChecked(nil);
		end
	end
	
end

-- *****************************************************************************
-- BeneCastOptionsCheckButton functions
-- *****************************************************************************

function BeneCastOptionsCheckButton_OnClick()

	local dontredraw = nil;
	if BeneCastSetup.currentoptionid == nil then
		BeneCastSetup.currentoptionid = 1;
	end
	local pf = BeneCastSetup.currentoptionid;
  if BeneCastProfiles[pf] == nil then
    BeneCastProfiles[pf] = {};
  end
  
	-- Set the BeneCastProfiles for the option to take on the value of the checkbutton
	for index, value in pairs (BeneCastOptionFrameCheckButtons) do
		if ( value.index == this:GetID() ) then
			BeneCastProfiles[pf][value.cvar] = this:GetChecked();
			-- If this is the tooltip option check to see if it's enabled
			-- If not disable the tooltip name option checkbutton
			if ( value.index == 5 ) then
				if ( this:GetChecked() ) then
					getglobal('BeneCastOptionCheckButton6'):Enable();
				else
					getglobal('BeneCastOptionCheckButton6'):Disable();
				end
			-- If this is the notification option check to see if it's enabled
			-- If not disable all sub notification options
			elseif ( value.index == 8 ) then
				if ( this:GetChecked() ) then
					getglobal('BeneCastOptionCheckButton9'):Enable();
					getglobal('BeneCastOptionCheckButton10'):Enable();
					getglobal('BeneCastOptionCheckButton11'):Enable();
					getglobal('BeneCastOptionCheckButton13'):Enable();
					getglobal('BeneCastOptionCheckButton14'):Enable();
					getglobal('BeneCastOptionCheckButton15'):Enable();
					getglobal('BeneCastOptionCheckButton16'):Enable();
					getglobal('BeneCastOptionCheckButton17'):Enable();
					getglobal('BeneCastOptionCheckButton18'):Enable();
					getglobal('BeneCastOptionCheckButton19'):Enable();
					getglobal('BeneCastOptionCheckButton21'):Enable();
				else
					getglobal('BeneCastOptionCheckButton9'):Disable();
					getglobal('BeneCastOptionCheckButton10'):Disable();
					getglobal('BeneCastOptionCheckButton11'):Disable();
					getglobal('BeneCastOptionCheckButton13'):Disable();
					getglobal('BeneCastOptionCheckButton14'):Disable();
					getglobal('BeneCastOptionCheckButton15'):Disable();
					getglobal('BeneCastOptionCheckButton16'):Disable();
					getglobal('BeneCastOptionCheckButton17'):Disable();
					getglobal('BeneCastOptionCheckButton18'):Disable();
					getglobal('BeneCastOptionCheckButton19'):Disable();
					getglobal('BeneCastOptionCheckButton21'):Disable();
				end
			-- If this is the heal notification option check to see if it's enabled
			-- If not disable all heal max notification
			elseif ( value.index == 16 ) then
				if ( this:GetChecked() ) then
					getglobal('BeneCastOptionCheckButton17'):Enable();
				else
					getglobal('BeneCastOptionCheckButton17'):Disable();
				end
			-- If this is the hide minimap button option check to see if it's enabled
			-- If so then hide the minimap button
			elseif ( value.index == 4 ) then
				if ( this:GetChecked() ) then
					getglobal('BeneCastMinimapButton'):Hide();
				else
					getglobal('BeneCastMinimapButton'):Show();
				end	
			--Allow Raidpets to be clicked only if Raidmembers is checked
			elseif ( value.index == 24 ) then
				local cbutton = getglobal('BeneCastOptionCheckButton25');
				if ( this:GetChecked() ) then
					cbutton:Enable();
				else
					BeneCastProfiles[pf].ShowRaidPets = false;
					cbutton:SetChecked(BeneCastProfiles[pf].ShowRaidPets);
					cbutton:Disable();
				end
				BC_UpdateRaid = true;
				dontredraw = true;
			--Redraw raidbuttons to show/hide raidpets
			elseif ( value.index == 25 ) then
				BC_UpdateRaid = true;
				dontredraw = true;
			--Redraw raidbuttons to overlap or not to overlap
			elseif ( value.index == 1 ) then
				BC_UpdateRaid = true;
				dontredraw = true;
			end
			do break end; -- Break for efficiency
		end
	end
	-- Play a sound for player feedback
	if ( this:GetChecked() ) then
		PlaySound('igMainMenuOptionCheckBoxOff');
	else
		PlaySound('igMainMenuOptionCheckBoxOn');
	end
	
	-- Redraw everyone's buttons
	if not dontredraw then
		if pf == BeneCastSetup.currentid then
			for i = 1, 5 do
				BeneCast_UpdateButtons(i);
			end
			for i = 46, 50 do
				BeneCast_UpdateButtons(i);
			end
			for i = 91, 93 do
				BeneCast_UpdateButtons(i);
			end
		end
		if pf == BeneCastSetup.currentraidid then
			for i = 6, 45 do
				BeneCast_UpdateButtons(i);
			end
			for i = 51, 90 do
				BeneCast_UpdateButtons(i);
			end
		end
	end

end

-- *****************************************************************************
-- BeneCastProfileCheckButton functions
-- *****************************************************************************

function BeneCastProfileCheckButton_OnClick()

	if not this:GetChecked() then
		this:SetChecked(true);
	else
		local profilebox = getglobal('BeneCastProfileName' .. this.var2);
		if profilebox:GetText() then
			local found;
			for k, v in pairs(BeneCastProfiles) do
				if v.profilename == profilebox:GetText() then
					found = k;
					do break end;
				end
			end
			if found then
				if this.var1 == 'profilecombat' then
					for i = 1, 5 do
						if i ~= this.var2 then
							getglobal('BeneCastCombatCheck' .. i):SetChecked(false);
						end
					end
					BeneCastSetup['PrefferedInCombatProfile'] = found;
				elseif this.var1 == 'profileoutofcombat' then
					for i = 1, 5 do
						if i ~= this.var2 then
							getglobal('BeneCastOutOfCombatCheck' .. i):SetChecked(false);
						end
					end
					BeneCastSetup['PrefferedOutOfCombatProfile'] = found;
				elseif this.var1 == 'profilecombatraid' then
					for i = 1, 5 do
						if i ~= this.var2 then
							getglobal('BeneCastCombatCheck' .. i .. 'r'):SetChecked(false);
						end
					end
					BeneCastSetup['PrefferedCombatProfileRaid'] = found;
				elseif this.var1 == 'profilecoutofombatraid' then
					for i = 1, 5 do
						if i ~= this.var2 then
							getglobal('BeneCastOutOfCombatCheck' .. i .. 'r'):SetChecked(false);
						end
					end
					BeneCastSetup['PrefferedOutOfCombatProfileRaid'] = found;
				end
				BeneCast_UpdateCurrentProfile();
			else
				PutMsg('No profile found with name=' .. profilebox:GetText());
				this:SetChecked(false);
			end
		else
			this:SetChecked(false);
		end
	end
				
end

function BeneCast_UpdateProfileCheckButtons()

	local profindex = 1;
	for k, v in pairs(BeneCastProfiles) do
		local profilebox = getglobal('BeneCastProfileName' .. profindex);
		profilebox:SetText(v.profilename);
		profilebox.oldtext = v.profilename;
		profindex = profindex + 1;
	end
	for i = profindex, 5 do
		local profilebox = getglobal('BeneCastProfileName' .. i);
		profilebox:SetText('');
		profilebox.oldtext = '';
	end

	for i = 1, 5 do
		local profilebox = getglobal('BeneCastProfileName' .. i);
		local checkbutton;
		checkbutton = getglobal('BeneCastCombatCheck' .. i);
		checkbutton:SetChecked(profilebox:GetText() == BeneCastSetup.PrefferedInCombatProfile);
		checkbutton = getglobal('BeneCastOutOfCombatCheck' .. i);
		checkbutton:SetChecked(profilebox:GetText() == BeneCastSetup.PrefferedOutOfCombatProfile);
		checkbutton = getglobal('BeneCastCombatCheck' .. i .. 'r');
		checkbutton:SetChecked(profilebox:GetText() == BeneCastSetup.PrefferedInCombatProfileRaid);
		checkbutton = getglobal('BeneCastOutOfCombatCheck' .. i .. 'r');
		checkbutton:SetChecked(profilebox:GetText() == BeneCastSetup.PrefferedOutOfCombatProfileRaid);
	end
end

function BeneCast_UpdateCurrentProfile()

	if BC_ConsiderInCombat then
		BeneCastSetup.currentid = BeneCastSetup.PrefferedInCombatProfile;
		BeneCastSetup.currentraidid = BeneCastSetup.PrefferedInCombatProfileRaid;
	else
		BeneCastSetup.currentid = BeneCastSetup.PrefferedOutOfCombatProfile;
		BeneCastSetup.currentraidid = BeneCastSetup.PrefferedOutOfCombatProfileRaid;
	end
	
	if BeneCastProfiles[BeneCastSetup.currentid] == nil then
	  BeneCastProfiles[BeneCastSetup.currentid] = {};
	end
	if BeneCastProfiles[BeneCastSetup.currentraidid] == nil then
	  BeneCastProfiles[BeneCastSetup.currentraidid] = {};
	end
	
	if BeneCastProfiles[BeneCastSetup.currentid] then
		for i = 1, 5 do
			BeneCast_UpdateButtons(i);
		end
		for i = 46, 50 do
			BeneCast_UpdateButtons(i);
		end
		for i = 91, 93 do
			BeneCast_UpdateButtons(i);
		end
	end
	if BeneCastProfiles[BeneCastSetup.currentraidid] then
		for i = 6, 45 do
			BeneCast_UpdateButtons(i);
		end
		for i = 51, 90 do
			BeneCast_UpdateButtons(i);
		end
	end
end

-- *****************************************************************************
-- BeneCastSpellCheckButton functions
-- *****************************************************************************

function BeneCastSpellCheckButton_OnClick()

	if BeneCastSetup.currentoptionid == nil then
		BeneCastSetup.currentoptionid = 1;
	end
	local profileid = BeneCastSetup.currentoptionid;
	if BeneCastProfiles[profileid] == nil then
	  BeneCastProfiles[profileid] = {};
	end
	for i = 1, 10 do
		if BeneCastProfiles[profileid][i] == nil then
			BeneCastProfiles[profileid][i] = {};
		end
	end

	-- Put some things in a local in the beginning for efficiency (thnx Shimavak)
	local spellname = getglobal( this:GetName() .. 'Text' ):GetText();
	local parentid = this:GetParent():GetID();
	
	-- Set the BeneCastProfiles for the option to take on the value of the checkbutton
	local spelltype;
	if ( spellname == BENECAST_STRINGS.TEXT_ASSIST ) then
		spelltype = 'assist';
	else
		spelltype = BeneCast_SpellTypes[BC_class][spellname];
	end
	BeneCastProfiles[profileid][parentid][spelltype] = this:GetChecked();
	if ( parentid == 1 and BeneCastProfiles[profileid]['PlayerAsDefault'] ) then
		for i = 2, 10 do
			BeneCastProfiles[profileid][i][spelltype] = this:GetChecked();
			getglobal(BeneCastOptionFrameSubframes[i] .. 'Button' .. this:GetID()):SetChecked(this:GetChecked());
		end
	end
	-- Play a sound for player feedback
	if ( this:GetChecked() ) then
		PlaySound('igMainMenuOptionCheckBoxOff');
	else
		PlaySound('igMainMenuOptionCheckBoxOn');
	end
	
	-- Redraw everyone's buttons
	for i in pairs (BC_targets) do
		BeneCast_UpdateButtons(i);
	end

end

-- *****************************************************************************
-- BeneCast_Tooltip functions
-- *****************************************************************************

function BeneCast_Tooltip_OnLoad()

	-- For some unknown reason, when doing reloadui, it will not parse the OnLoad from
	-- 	the XML file for BeneCast_Tooltip...so we're doing that here.
	this:SetOwner(WorldFrame, "ANCHOR_NONE");
end

-- *****************************************************************************
-- BeneCastPanelManager functions
-- *****************************************************************************

function BeneCastPanelManager_OnLoad()

	-- Set BC_class to the player's class
	BC_class = UnitClass('player');
	
	-- Register for Events
	this:RegisterEvent('ADDON_LOADED');
	this:RegisterEvent('PLAYER_ENTERING_WORLD');
	this:RegisterEvent('PLAYER_LEAVING_WORLD');
	this:RegisterEvent('VARIABLES_LOADED');
	this:RegisterEvent('SPELLS_CHANGED');
	this:RegisterEvent('SPELLCAST_FAILED');
	this:RegisterEvent('SPELLCAST_INTERRUPTED');
	this:RegisterEvent('SPELLCAST_STOP');
	this:RegisterEvent('SPELLCAST_START');
	this:RegisterEvent('UNIT_SPELLCAST_CHANNEL_START');
	this:RegisterEvent('UNIT_SPELLCAST_CHANNEL_STOP');
	this:RegisterEvent('PARTY_MEMBERS_CHANGED');
	this:RegisterEvent('PLAYER_TARGET_CHANGED');
	this:RegisterEvent('PLAYER_FOCUS_CHANGED');
	this:RegisterEvent('UNIT_AURA');
	this:RegisterEvent('PLAYER_AURAS_CHANGED');
	this:RegisterEvent('UNIT_AURASTATE');
	this:RegisterEvent('UPDATE_SHAPESHIFT_FORMS');
	this:RegisterEvent('PLAYER_ALIVE');
	this:RegisterEvent('PLAYER_DEAD');
	this:RegisterEvent('PLAYER_UNGHOST');
	this:RegisterEvent('RAID_ROSTER_UPDATE');
	this:RegisterEvent('UNIT_HEALTH');
	this:RegisterEvent('UNIT_PET');
	this:RegisterEvent('UNIT_TARGET');
	this:RegisterEvent('PLAYER_REGEN_DISABLED');
	this:RegisterEvent('PLAYER_REGEN_ENABLED');

end

function BeneCastPanelManager_OnEvent()

	if ( event == 'ADDON_LOADED' and arg1 == 'BeneCast' ) then
		BeneCast.db = LibStub("AceDB-3.0"):New("MyAddonDB");
		LibStub("AceConfig-3.0"):RegisterOptionsTable("BeneCast", options, {"benecast", "bc"});
			
		-- Reset the data if there BeneCast is severly outdated
		if ( not BeneCastSetup or not BeneCastProfiles or
		     table.getn(BeneCastProfiles) == 0 or
		     not BeneCastSetup.Version or BeneCastSetup.Version < BENECAST_VERSION ) then
			PutMsg('Resetting settings')
			BeneCast_ResetProfiles();
		end
		BeneCast_UpdateCurrentProfile();
		
		-- Populating all of our lists of constants now so we don't have to do it repeatedly.
		for i in pairs (BC_targets) do
			BC_attached_targets[i] = false;
		end
		
		for i,buffInfo in pairs (BC_buffs_in_effect) do
			BC_buffs_in_effect_by_name[buffInfo.name] = i;
		end
		
		-- Load up spell data once AddOn has been loaded
		BeneCast_ResetData();
		
		--Code from BeneCastOptionFrame
		BeneCast_LoadOptions();
		BeneCast_UpdateProfileCheckButtons();
		BeneCast_ResizeButtons();
		BeneCast_ClearRaid();
		
		BENECAST_RAID_LIST = {};
				
	elseif ( event == 'ADDON_LOADED' and ( arg1 == 'Blizzard Raid UI' or arg1 == 'CT_RaidAssist' ) ) then
		if arg1 == 'CT_RaidAssist' then
			--Hook into CT_RA's update function
			if BC_Old_CTRA_UpdateFunction == nil and CT_RA_UpdateRaidGroup ~= nil then
				BC_Old_CTRA_UpdateFunction = CT_RA_UpdateRaidGroup;
				CT_RA_UpdateRaidGroup = BC_UpdateRaidGroup;
			end
		end
			
		BC_UpdateRaid = true;
	
	elseif ( event == 'ADDON_LOADED' ) then
		-- Load up the variables for the snapping if empty
		if BeneCastSetup.currentid == nil then
			BeneCastSetup.currentid = 1;
		end
		if BeneCastProfiles[BeneCastSetup.currentid] == nil then
			BeneCastProfiles[BeneCastSetup.currentid] = { SnapTo = 'BeneCast' };
		end
		if BeneCastProfiles[BeneCastSetup.currentid].SnapTo ~= 'CUSTOM' then
			BeneCast_SnapPanels(arg1);
		end
		
	elseif ( event == 'VARIABLES_LOADED' ) then
		if BeneCastSetup.currentid == nil then
			BeneCastSetup.currentid = 1;
		end
		if BeneCastProfiles[BeneCastSetup.currentid] == nil then
			BeneCastProfiles[BeneCastSetup.currentid] = {};
		end
		BeneCast_SnapPanels(BeneCastProfiles[BeneCastSetup.currentid].SnapTo);
		-- Wait with the snapping till everything is loaded
		BC_AttachPartyFrames = true;
	
	-- If SPELLCAST_FAILED then a BeneCast spell may have failed
	elseif ( event == 'SPELLCAST_FAILED' ) then
		-- A BeneCast spell failed if a BC_spellstarted and BC_sentmessage
		if ( BC_spellstarted and BC_sentmessage ) then
			BeneCast_PartyNotifyFail('SPELLCAST_FAILED');
		end
		-- Clear out everything used for party notification just in case
		BC_spellstarted = nil;
		BC_spellstopped = nil;
		BC_spelltarget = nil;
		BC_spellbeingcast = nil;
		BC_spellismax = nil;
		BC_sentmessage = nil;
		
	-- If SPELLCAST_INTERRUPTED then a BeneCast spell may have been interrupted
	elseif ( event == 'SPELLCAST_INTERRUPTED' ) then
		-- A BeneCast spell was interrupted by the player if BC_spellstopped and BC_sentmessage
		if ( BC_spellstopped and BC_sentmessage ) then
			BeneCast_PartyNotifyFail('SPELLCAST_CANCELED');
		end
		-- A BeneCast spell was interrupted by something else if BC_spellstarted and BC_sentmessage
		if ( BC_spellstarted and BC_sentmessage ) then
			BeneCast_PartyNotifyFail('SPELLCAST_INTERRUPTED');
		end
		-- Clear out everything just in case
		BC_spellstarted = nil;
		BC_spellstopped = nil;
		BC_spelltarget = nil;
		BC_spellbeingcast = nil;
		BC_spellismax = nil;
		BC_sentmessage = nil;
		
	elseif ( event == 'SPELLCAST_STOP' or event == 'UNIT_SPELLCAST_CHANNEL_STOP' ) then
		-- If BC_spellbeingcast and notification is on then this is an instant spell and you should do notification
		if ( BC_spellbeingcast ) then
			-- BC_sentmessage will be set to true of BeneCast_PartyNotify sends a message
			BC_sentmessage = BeneCast_PartyNotify(BC_spelltarget, BC_spellbeingcast, BC_spellismax, 0);
		-- Otherwise set _setmessage to false
		else
			BC_sentmessage = nil
		end
		-- If BC_spellstarted, then a BeneCast spell was started and you should set BC_sentmessage and BC_spellstopped to true to indicate that the spell was interrupted by the user
		if ( BC_spellstarted ) then
			BC_sentmessage = true;
			BC_spellstopped = true;
			BC_spellstarted = nil;
		else
			-- Clear this, just in case
			BC_spellstopped = nil;
		end
		-- Clear these out just in case
		BC_spelltarget = nil;
		BC_spellbeingcast = nil;
		BC_spellismax = nil;
		
	elseif ( event == 'SPELLCAST_START' or event == 'UNIT_SPELLCAST_CHANNEL_START' ) then
		-- If the spell being cast exists in BeneCast_SpellTypes, notification is on, and there is a BC_spellbeingcast then do notification
		if ( BeneCast_SpellTypes[BC_class][arg1] and BC_spellbeingcast ) then
			-- Set BC_sentmessage to true of BeneCast_PartyNotify sent a message
			BC_sentmessage = BeneCast_PartyNotify(BC_spelltarget, BC_spellbeingcast, BC_spellismax, arg2);
			-- Clear out BC_spelltarget, BC_spellbeingcast, and BC_spellismax so it isn't used by SPELLCAST_STOP
			BC_spelltarget = nil;
			BC_spellbeingcast = nil;
			BC_spellismax = nil;
			-- If you sent a message then BC_spellstarted to indicate failures/interruptions should send notifications
			if ( BC_sentmessage ) then
				BC_spellstarted = true;
			end
		else
			-- Clear out everything just in case
			BC_spellstarted = nil;
			BC_spellstopped = nil;
			BC_spelltarget = nil;
			BC_spellbeingcast = nil;
			BC_spellismax = nil;
			BC_sentmessage = nil;
		end
	
	-- We only want spell changes after entering the world
	elseif ( event == 'SPELLS_CHANGED' and arg1 == nil) then -- don't fire when user only browses spellbook
		
		-- Reset most of the tables for reloading of spells and reload the spell data
		BeneCast_ResetData();
		for i in pairs (BC_targets) do
			BeneCast_UpdateButtons(i);
		end
	
	-- Don't bother with UNIT_AURA if the player can attack the unit
	elseif ( ( event == 'UNIT_AURA' or event == 'UNIT_AURASTATE' ) and not UnitCanAttack('player', arg1) ) then
		if BC_AttachPartyFrames then
			BeneCast_AttachPartyPanelsFromVar();
			BC_AttachPartyFrames = nil;
		end
		-- Redraw the buttons for the UnitID in arg1
		if BeneCastSetup.Debug then
			local msg = arg1;
			if not msg then
				msg = 'nil';
			end
			PutDebugMsg(event .. ' for ' .. msg);
		end
		BeneCast_UpdateButtonsForUnit(arg1);
		
	elseif ( event == 'PLAYER_AURAS_CHANGED' ) then
		if BC_AttachPartyFrames then
			BeneCast_AttachPartyPanelsFromVar();
			BC_AttachPartyFrames = nil;
		end
		BeneCast_UpdateButtonsForUnit('player');
	
	elseif ( event == 'PARTY_MEMBERS_CHANGED' ) then
		-- Redraw the buttons for all partymembers (and partypets)
		for i = 1,4 do
			BC_UnitUpdater['party' .. i] = GetTime() + 1;
			BC_UnitUpdater['partypet' .. i] = GetTime() + 1;
		end
	
	-- Fired when the pet of a unit changes
	elseif ( event == 'UNIT_PET' ) then
		if arg1 == 'player' then
			BC_UnitUpdater['pet'] = GetTime() + 1;
		elseif string.find(arg1,'party') then
			local num;
			_, _, num = string.find(arg1,'%d+');
			if num then
				BC_UnitUpdater['partypet' .. num] = GetTime() + 1;
			end
		elseif string.find(arg1,'raid') then
			if BeneCastSetup.currentraidid then
				if BeneCastProfiles[BeneCastSetup.currentraidid] then
					if BeneCastProfiles[BeneCastSetup.currentraidid].ShowRaidPets then
						local num;
						_, _, num = string.find(arg1,'%d+');
						if num then
							BC_UnitUpdater['raidpet' .. num] = GetTime() + 1;
						end
					end
				end
			end
		end
		
	elseif ( event == 'PLAYER_TARGET_CHANGED' ) then
		PutDebugMsg('PLAYER_TARGET_CHANGED');
		BeneCast_AttachPartyPanelsFromVar();
		BC_AttachPartyFrames = nil;
		-- Redraw the buttons for the target
		BeneCast_UpdateButtonsForUnit('target');
		BC_UnitUpdater['target'] = GetTime() + 1;
		-- Redraw the buttons for the target of target
		BeneCast_UpdateButtonsForUnit('targettarget');
		BC_UnitUpdater['targettarget'] = GetTime() + 1;

	elseif ( event == 'PLAYER_FOCUS_CHANGED' ) then
		-- Redraw the buttons for the target
		BeneCast_AttachPanelsFromVar(93);
		BeneCast_UpdateButtonsForUnit('focus');
		BC_UnitUpdater['focus'] = GetTime() + 1;
	
	elseif ( event == 'UNIT_TARGET' and unit == 'target' ) then
		PutDebugMsg('UNIT_TARGET');
		-- Redraw the buttons for the target of target
		BeneCast_AttachPanelsFromVar(92);
		BeneCast_UpdateButtonsForUnit('targettarget');
		BC_UnitUpdater['targettarget'] = GetTime() + 1;
		
	elseif ( event == 'RAID_ROSTER_UPDATE' ) then
		PutDebugMsg('RAID_ROSTER_UPDATE event received');
		if not BeneCastSetup.currentraidid then
			BeneCastSetup.currentraidid = 1
		end
		local pf = BeneCastSetup.currentraidid;
		if BeneCastProfiles[pf] == nil then
			BeneCastProfiles[pf] = {};
		end
		if BeneCastProfiles[pf].RaidSnapTo == 'STANDARDBOTTOM' or BeneCastProfiles[pf].RaidSnapTo == 'STANDARDRIGHT' then
			BC_UpdateRaid = true;
		end
				
	elseif ( event == 'PLAYER_ALIVE' or event == 'PLAYER_DEAD' or event == 'PLAYER_UNGHOST' or event == 'UPDATE_SHAPESHIFT_FORMS' ) then
		-- These changes shouldn't affect who can be shown...
		-- so we only want to update those that we've been showing before. 
		for i in pairs (BC_targets) do
			BeneCast_UpdateButtons(i);
		end
		
	elseif ( event == 'UNIT_HEALTH' ) then
		-- Redraw a member's panel if they're dead
		if ( UnitHealth(arg1) <= 0 and UnitIsConnected(arg1) ) then
			BeneCast_UpdateButtonsForUnit(arg1);
		end
				
	elseif ( event == 'PLAYER_ENTERING_WORLD' ) then
		BeneCast_AttachPartyPanelsFromVar();
		this:RegisterEvent('ACTIONBAR_SLOT_CHANGED');
		this:RegisterEvent('SPELLS_CHANGED');
		this:RegisterEvent('SPELLCAST_FAILED');
		this:RegisterEvent('SPELLCAST_INTERRUPTED');
		this:RegisterEvent('SPELLCAST_STOP');
		this:RegisterEvent('SPELLCAST_START');
		this:RegisterEvent('PARTY_MEMBERS_CHANGED');
		this:RegisterEvent('PLAYER_TARGET_CHANGED');
		this:RegisterEvent('UNIT_AURA');
		this:RegisterEvent('PLAYER_AURAS_CHANGED');
		this:RegisterEvent('UNIT_AURASTATE');
		this:RegisterEvent('UPDATE_SHAPESHIFT_FORMS');
		this:RegisterEvent('PLAYER_ALIVE');
		this:RegisterEvent('PLAYER_DEAD');
		this:RegisterEvent('PLAYER_UNGHOST');
		this:RegisterEvent('RAID_ROSTER_UPDATE');
		this:RegisterEvent('UNIT_HEALTH');
		this:RegisterEvent('UNIT_PET');
	
	elseif ( event == 'PLAYER_LEAVING_WORLD' ) then
		this:UnregisterEvent('ACTIONBAR_SLOT_CHANGED');
		this:UnregisterEvent('SPELLS_CHANGED');
		this:UnregisterEvent('SPELLCAST_FAILED');
		this:UnregisterEvent('SPELLCAST_INTERRUPTED');
		this:UnregisterEvent('SPELLCAST_STOP');
		this:UnregisterEvent('SPELLCAST_START');
		this:UnregisterEvent('PARTY_MEMBERS_CHANGED');
		this:UnregisterEvent('PLAYER_TARGET_CHANGED');
		this:UnregisterEvent('UNIT_AURA');
		this:UnregisterEvent('PLAYER_AURAS_CHANGED');
		this:UnregisterEvent('UNIT_AURASTATE');
		this:UnregisterEvent('UPDATE_SHAPESHIFT_FORMS');
		this:UnregisterEvent('PLAYER_ALIVE');
		this:UnregisterEvent('PLAYER_DEAD');
		this:UnregisterEvent('PLAYER_UNGHOST');
		this:UnregisterEvent('RAID_ROSTER_UPDATE');
		this:UnregisterEvent('UNIT_HEALTH');
		this:UnregisterEvent('UNIT_PET');
	
	elseif ( event == 'PLAYER_REGEN_DISABLED' ) then
		-- Player entering combat
		BC_ConsiderInCombat = true;
		BeneCast_UpdateCurrentProfile();
		BeneCast_LoadOptions();
	
	elseif ( event == 'PLAYER_REGEN_ENABLED' ) then
		-- Player exiting combat
		BC_ConsiderInCombat = false;
		BeneCast_UpdateCurrentProfile();
		BeneCast_LoadOptions();
	
	end

end

function BeneCastPanelManager_OnUpdate()

	local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges;
	
	hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo();
	
	if ( hasMainHandEnchant ) then
		local slotId, textureName;
	
		slotId, textureName = GetInventorySlotInfo('MainHandSlot');
	
		BeneCast_Tooltip:ClearLines();
		BeneCast_Tooltip:SetInventoryItem('player', slotId);
		
		for i = 1, BeneCast_Tooltip:NumLines() do
			if getglobal('BeneCast_TooltipTextLeft' .. i):GetText() then
				local line = getglobal('BeneCast_TooltipTextLeft' .. i):GetText();
				if ( BC_weaponenchant ~= BENECAST_STRINGS.WEAPONENCHANT_ROCKBITER and string.find(line, BENECAST_STRINGS.WEAPONENCHANT_ROCKBITER) ) then
					BC_weaponenchant = BENECAST_STRINGS.WEAPONENCHANT_ROCKBITER;
					BeneCast_UpdateButtons(1);
					do break end; --WINTROW.6, break for efficiency
				end
				if ( BC_weaponenchant ~= BENECAST_STRINGS.WEAPONENCHANT_FLAMETONGUE and string.find(line, BENECAST_STRINGS.WEAPONENCHANT_FLAMETONGUE) ) then
					BC_weaponenchant = BENECAST_STRINGS.WEAPONENCHANT_FLAMETONGUE;
					BeneCast_UpdateButtons(1);
					do break end; --WINTROW.6, break for efficiency
				end
				if ( BC_weaponenchant ~= BENECAST_STRINGS.WEAPONENCHANT_FROSTBRAND and string.find(line, BENECAST_STRINGS.WEAPONENCHANT_FROSTBRAND) ) then
					BC_weaponenchant = BENECAST_STRINGS.WEAPONENCHANT_FROSTBRAND;
					BeneCast_UpdateButtons(1);
					do break end; --WINTROW.6, break for efficiency
				end
				if ( BC_weaponenchant ~= BENECAST_STRINGS.WEAPONENCHANT_WINDFURY and string.find(line, BENECAST_STRINGS.WEAPONENCHANT_WINDFURY) ) then
					BC_weaponenchant = BENECAST_STRINGS.WEAPONENCHANT_WINDFURY;
					BeneCast_UpdateButtons(1);
					do break end; --WINTROW.6, break for efficiency
				end
				if ( BC_weaponenchant ~= BENECAST_STRINGS.WEAPONENCHANT_FEEDBACK and string.find(line, BENECAST_STRINGS.WEAPONENCHANT_FEEDBACK) ) then
					BC_weaponenchant = BENECAST_STRINGS.WEAPONENCHANT_FEEDBACK;
					BeneCast_UpdateButtons(1);
					do break end; --WINTROW.6, break for efficiency
				end
			end
		end
	else
		if ( BC_weaponenchant ) then
			BC_weaponenchant = nil;
			BeneCast_UpdateButtons(1);
		end
		BC_weaponenchant = nil;
	end

	-- Update all button-flashing at once
	local buffAlphaValue = nil;
	for i,buttonname in pairs (BC_buttons_that_flash) do
		local button = getglobal(buttonname);
		if button then
			if buffAlphaValue == nil then --this way it only has to be calculated once
				if ( BuffFrameFlashState == 1 ) then
					buffAlphaValue = (BUFF_FLASH_TIME_ON - BuffFrameFlashTime) / BUFF_FLASH_TIME_ON;
					buffAlphaValue = buffAlphaValue * (1 - BUFF_MIN_ALPHA) + BUFF_MIN_ALPHA;
				else
					buffAlphaValue = BuffFrameFlashTime / BUFF_FLASH_TIME_ON;
					buffAlphaValue = (buffAlphaValue * (1 - BUFF_MIN_ALPHA)) + BUFF_MIN_ALPHA;
					--this:SetAlpha(BuffFrameFlashTime / BUFF_FLASH_TIME_ON);
				end
			end
			button:SetAlpha(buffAlphaValue);
		end
	end
	
	if not BeneCastSetup.currentraidid then
		BeneCastSetup.currentraidid = 1;
	end
	local pf = BeneCastSetup.currentraidid;
	if BeneCastProfiles[pf] == nil then
		BeneCastProfiles[pf] = {};
	end
	if BeneCastProfiles[pf].ShowRaid then
		local docheck = false;
		if BC_UpdateRaid then
			if BC_Last_UpdateRaid_Done == nil then
				docheck = true;
			elseif (GetTime() - BC_Last_UpdateRaid_Done) >= 5 then
				docheck = true;
			end
		end
		if docheck then
			BC_Last_UpdateRaid_Done = GetTime();
			BC_UpdateRaid = nil;
			BeneCast_UpdateRaid();
		end
	end
		
	-- Check for buttons if they are within range
	if not BeneCastSetup.currentid then
		BeneCastSetup.currentid = 1;
	end
	local pf = BeneCastSetup.currentid;
	if BeneCastProfiles[pf] == nil then
		BeneCastProfiles[pf] = {};
	end
	if BeneCastProfiles[pf].CheckRange then
		local docheck = false;
		if ( BC_LastRangeCheck == nil ) or
		   ( (GetTime() - BC_LastRangeCheck) > 1 ) then -- Every 1 second
	   		docheck = true;
		end
		if docheck then
			BC_LastRangeCheck = GetTime();
			BeneCast_CheckRange();
		end
	end
	
	-- Check if any panels need updating
	local todelete = {};
	for unit, updatetime in pairs(BC_UnitUpdater) do
		if updatetime and (GetTime() > updatetime) then
			-- can't remove items while traversing the table
			-- it would mess things up
			table.insert(todelete,unit); 
			BeneCast_UpdateButtonsForUnit(unit);
		end
	end
	for k, unit in pairs(todelete) do
		BC_UnitUpdater[unit] = nil;
	end

end

function BeneCast_CheckRange()

	local inrange;
	for id = 2, table.getn(BC_targets) do
		local unit = BeneCast_GetUnitIDFromPanelID(id);
		if getglobal('BeneCastPanel' .. id):IsVisible() and UnitExists(unit) then
			for i = 1, BENECAST_MAX_NUMBER_OF_BUTTONS do
				local buttonname = 'BeneCastPanel' .. id .. 'Button' .. i;
				local button = getglobal(buttonname);
				if button:IsVisible() then
					local spellid = button.spellid;
					local range = getglobal(buttonname .. 'Range');
					range:Hide();
					if spellid then
						inrange = IsSpellInRange(spellid,unit);
						if (inrange == nil) or (inrange == 0) then
							range:Show();
							if unit == 'target' and BeneCastSetup.Debug then
								local spellname = GetSpellName(spellid, SpellBookFrame.bookType);
								PutDebugMsg('Button ' .. i .. ', spell ' .. spellname .. ' out of range');
							end
						end
					end
				end
			end
		end
	end

end

-- *****************************************************************************
-- BeneCastResetButton functions
-- *****************************************************************************

function BeneCast_ResetTooltip()

	GameTooltip:SetOwner(this, 'ANCHOR_RIGHT');
	GameTooltip:SetText(BENECAST_STRINGS['TEXT_RESET_BUTTON_TOOLTIP'], nil, nil, nil, nil, 1);

end

function BeneCast_ResetProfiles()

	--Reset the BeneCastProfiles-table
	if not BeneCastSetup then
		BeneCastSetup = {};
	end
	BeneCastProfiles = {};
	BeneCastProfiles[1] = { profilename = '<default>' };
	BeneCastSetup.currentid = 1;
	BeneCastSetup.currentraidid = 1;
	BeneCastSetup.currentoptionid = 1;
	BeneCastSetup.PrefferedInCombatProfile = 1;
	BeneCastSetup.PrefferedOutOfCombatProfile = 1;
	BeneCastSetup.Version = BENECAST_VERSION;
	
	BeneCast_ResetConfig(1);
end

function BeneCast.ResetConfig(id)

	-- Reset the specified Profile table
	local curid = BeneCastSetup.currentoptionid;
	if id == nil then
		id = curid
	end
	if BeneCastProfiles[id] == nil then
		BeneCastProfiles[id] = {};
	end
	
	local profname = BeneCastProfiles[id].profilename;
	BeneCastProfiles[id] = {};
	if profname == nil then
		profname = 'profile' .. id;
	end 
	BeneCastProfiles[id].profilename = profname;
	
	BeneCastProfiles[id][1] = {};
	BeneCastProfiles[id][2] = {};
	BeneCastProfiles[id][3] = {};
	BeneCastProfiles[id][4] = {};
	BeneCastProfiles[id][5] = {};
	BeneCastProfiles[id][6] = {};
	BeneCastProfiles[id][7] = {};
	BeneCastProfiles[id][8] = {};
	BeneCastProfiles[id][9] = {};
	BeneCastProfiles[id][10] = {};

	for i = 1,10 do
		for type in pairs (BeneCast_SpellTypes_Sorting[BC_class]) do
			BeneCastProfiles[id][i][type] = false;
		end
	end
	
	BeneCastProfiles[id]['ShowAllButtons'] = false;
	BeneCastProfiles[id]['OverlapRaidBars'] = false;
	BeneCastProfiles[id]['FlashAsFade'] = false;
	BeneCastProfiles[id]['HideMinimap'] = false;
	BeneCastProfiles[id]['ShowTooltips'] = true;
	BeneCastProfiles[id]['ShowTooltipsName'] = false;
	BeneCastProfiles[id]['Unlock'] = false;
	BeneCastProfiles[id]['Notification'] = false;
	BeneCastProfiles[id]['SelfNotify'] = true;
	BeneCastProfiles[id]['PartyNotify'] = false;
	BeneCastProfiles[id]['SayNotify'] = false;
	BeneCastProfiles[id]['RaidNotify'] = false;
	BeneCastProfiles[id]['TargetNotify'] = false;
	BeneCastProfiles[id]['UserNotify'] = false;
	BeneCastProfiles[id]['SelfCasts'] = true;
	BeneCastProfiles[id]['NotifyRank'] = true;
	BeneCastProfiles[id]['NotifyTime'] = true;
	BeneCastProfiles[id]['NotifyHeal'] = true;
	BeneCastProfiles[id]['NotifyMaxHeal'] = false;
	BeneCastProfiles[id]['NotifyCure'] = true;
	BeneCastProfiles[id]['NotifyBuff'] = true;
	BeneCastProfiles[id]['NotifyRes'] = true;
	BeneCastProfiles[id]['PlayerAsDefault'] = true;
	BeneCastProfiles[id]['ShowPets'] = false;
	BeneCastProfiles[id]['ShowRaid'] = false;
	BeneCastProfiles[id]['ShowRaidPets'] = false; 
	BeneCastProfiles[id]['CheckRange'] = false; 
	BeneCastProfiles[id]['ShowTargetTarget'] = true; 
	
	BeneCastProfiles[id]['ButtonSize'] = 36;
	BeneCastProfiles[id]['ButtonNumber'] = 5;
	
	BeneCastProfiles[id]['UserChannel'] = nil;
	
	BeneCastProfiles[id]['SnapTo'] = 'STANDARD';
	BeneCastProfiles[id]['SnapToBackup'] = 'STANDARD';
	BeneCastProfiles[id]['RaidSnapTo'] = 'STANDARDRIGHT';
	
	BeneCastProfiles[id]['Debug'] = false;
	
	if id == curid then
		BeneCast_ResetData();
		BeneCast_LoadOptions();
		-- Reset the SnapTo selections (Raid too, thnx Shimavak)
		UIDropDownMenu_SetSelectedValue(BeneCastSnapToMenu, BeneCastProfiles[id].SnapTo);
		UIDropDownMenu_SetSelectedValue(BeneCastRaidSnapToMenu, BeneCastProfiles[id].RaidSnapTo);
	end
	if id == BeneCastSetup.currentid then
		-- Place the panels in the correct place
		BeneCast_SnapPanels(BeneCastProfiles[id].SnapTo);
		BeneCast_AttachPartyPanelsFromVar();
	
		-- Redraw everyone's panels
		--players
		for i = 1, 5 do
			BeneCast_OrientButtons(i);
			BeneCast_UpdateButtons(i);
		end
		--pets
		for i = 46, 50 do
			BeneCast_OrientButtons(i);
			BeneCast_UpdateButtons(i);
		end
		--target, targettarget, focus
		for i = 91, 93 do
			BeneCast_OrientButtons(i);
			BeneCast_UpdateButtons(i);
		end
	end

end

-- *****************************************************************************
-- BeneCastUserChannel functions
-- *****************************************************************************

function BeneCastUserChannel_OnTextChange()

	if BeneCastSetup.currentoptionid == nil then
		BeneCastSetup.currentoptionid = 1;
	end
	local pf = BeneCastSetup.currentoptionid;
	if BeneCastProfiles[pf] == nil then
		BeneCastProfiles[pf] = {};
	end
	BeneCastProfiles[pf]['UserChannel'] = this:GetText();

end

-- *****************************************************************************
-- BeneCastEditBox functions
-- *****************************************************************************

function BeneCastEditBox_ApplyChange()
	
	if this.var1 == 'variable' then
		if this.var2 == 'BC_PanelNumber' then
			BC_PanelNumber = tonumber(this:GetText());
			BeneCastUpdateCustomEditBoxes();
		end
		
	elseif this.var1 == 'panelinfo' then
		local pf = BeneCast_ProfileForPanelID(BC_PanelNumber);
		local framename = 'BeneCastPanel' .. BC_PanelNumber;
		if BeneCastSetup.Debug then
			local msg = 'Updating BeneCastProfiles[' .. pf .. '][';
			if framename then
				msg = msg .. framename .. '][';
			else
				msg = msg .. 'nil][';
			end
			if this.var2 then
				msg = msg .. this.var2 .. ']';
			else
				msg = msg .. 'nil]';
			end
			PutDebugMsg(msg);
		end
		local boxvalue = this:GetText();
		if boxvalue ~= BeneCastProfiles[pf][framename][this.var2] then
			BeneCastProfiles[pf]['SnapTo'] = 'CUSTOM';
			UIDropDownMenu_SetSelectedValue(BeneCastSnapToMenu, 'CUSTOM');
			BeneCastProfiles[pf][framename][this.var2] = boxvalue;
			BeneCastProfiles[pf][framename]['Moved'] = false;
		end
		PutDebugMsg('Updating Panel ' .. BC_PanelNumber);
		BeneCast_AttachPanelsFromVar(tonumber(BC_PanelNumber));
		
	elseif this.var1 == 'profilename' then
		local newtext = this:GetText();
		if nextext == '' then
			newtext = nil;
		end
		if newtext ~= this.oldtext then
			local profileid;
			if this.oldtext then
				for k,v in pairs(BeneCastProfiles) do
					if v.profilename == this.oldtext then
						profileid = k;
						do break end;
					end
				end
			end
			local newid;
			if newtext then
				for k,v in pairs(BeneCastProfiles) do
					if v.profilename == newtext then
						newid = k;
						do break end;
					end
				end
			end
			if newid then
				PutMsg('There is already a profile with that name');
				this:SetText(this.oldtext);
				return;
			end
			-- Change name of existing profile
			if profileid and newtext then
				BeneCastProfiles[profileid].profilename = newtext;
			-- Create new profile
			elseif not profileid and newtext then
				newid = table.getn(BeneCastProfiles)+1;
				BeneCastProfiles[newid] = {};
				BeneCastProfiles[newid].profilename = newtext;
				BeneCast_ResetConfig(newid);
				
			-- Delete existing profile
			elseif profileid and not newtext then
				table.remove(BeneCastProfiles,profileid);
				if table.getn(BeneCastProfiles) == 0 then
					BeneCast_ResetProfiles();
				end
				for i, v in pairs(BeneCastProfiles) do
					newid = i;
					do break end;
				end
				if BeneCastSetup['PrefferedInCombatProfile'] == profileid then
					BeneCastSetup['PrefferedInCombatProfile'] = newid;
				end
				if BeneCastSetup['PrefferedOutOfCombatProfile'] == profileid then
					BeneCastSetup['PrefferedOutOfCombatProfile'] = newid;
				end
				if BeneCastSetup.currentid == profileid then
					BeneCastSetup.currentid = nil;
					BeneCast_UpdateCurrentProfile();
				end
				BeneCast_LoadOptions();
			end
			BeneCast_UpdateProfileCheckButtons();
			this.oldtext = newtext;
		end
	
	elseif this.var1 == 'currprofile' then
		if this:GetText() == nil then
			this:SetText('1');
		elseif this:GetText() == '' then
			this:SetText('1');
		end
		local newid = tonumber(this:GetText());
		if newid ~= BeneCastSetup.currentoptionid then
			BeneCastSetup.currentoptionid = newid;
			BeneCast_LoadOptions();
			BeneCast_LoadSpellConfig();
		end
		
	end
	
end

-- *****************************************************************************
-- BeneCastSnapToMenu functions
-- *****************************************************************************

function BeneCastSnapToMenu_Initialize()
	
	local info;
	-- Add SnapTo options
	if BeneCastSetup.currentoptionid == nil then
		BeneCastSetup.currentoptionid = 1;
	end
	local pf = BeneCastSetup.currentoptionid;
	if BeneCastProfiles[pf] == nil then
		BeneCastProfiles[pf] = {};
	end
	for snapto in pairs (BeneCast_SnapTo) do
		if IsAddOnLoaded(BeneCast_SnapTo[snapto].AddOn) or snapto == 'CUSTOM' then
			info = {};
			if ( BeneCast_SnapTo[snapto].AddOn == 'BeneCast' ) then
				info.text = BENECAST_STRINGS.TEXT_STANDARD;
			elseif ( BeneCast_SnapTo[snapto].AddOn == 'Perl_Target' ) then
				info.text = BENECAST_STRINGS.TEXT_PERLCLASSIC;
			elseif ( snapto == 'CUSTOM' ) then
				info.text = 'Custom';
			else
				info.text = BeneCast_SnapTo[snapto].AddOn;
			end
			
			info.value = snapto;
			if ( BeneCastProfiles[pf].SnapTo == snapto ) then
				info.checked = true;
			else
				info.checked = false;
			end
			info.func = BeneCastSnapToMenu_SetSnap;
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
		end
	end

end

function SetBeneCastSnapTo(snapto_option)
	local found;
	for snaptoval in pairs (BeneCast_SnapTo) do
		if BeneCast_SnapTo[snaptoval].AddOn == snapto_option then
			if BeneCastSetup.currentoptionid == nil then
				BeneCastSetup.currentoptionid = 1;
			end
			local pf = BeneCastSetup.currentoptionid;
			if BeneCastProfiles[pf] == nil then
				BeneCastProfiles[pf] = {};
			end
			found = true;
			BeneCastProfiles[pf].SnapTo = snaptoval;
			if snaptoval ~= 'CUSTOM' then
				BeneCastProfiles[pf].SnapToBackup = snaptoval;
			end
			BeneCast_SnapPanels(snapto_option);
			BeneCast_AttachPartyPanelsFromVar(); --WINTROW.6
			UIDropDownMenu_SetSelectedValue(BeneCastSnapToMenu, snaptoval);
			for i in pairs (BC_targets) do
				BeneCast_UpdateButtons(i);
			end
			do break end;
		end
	end
	if not found then
		PutDebugMsg(snapto_option .. ' not found in predefined unit frames');
	end
end

function SetBeneCastSnapTo2(snapto_option)
	if ( not BeneCast_SnapTo[snapto_option] ) then
		PutDebugMsg(snapto_option .. ' not found in predefined unit frames');
		return;
	end
	if BeneCastSetup.currentoptionid == nil then
		BeneCastSetup.currentoptionid = 1;
	end
	local pf = BeneCastSetup.currentoptionid;
	if BeneCastProfiles[pf] == nil then
		BeneCastProfiles[pf] = {};
	end
	BeneCastProfiles[pf].SnapTo = snapto_option;
	if snapto_option ~= 'CUSTOM' then
		BeneCastProfiles[pf].SnapToBackup = snapto_option;
	end
	BeneCast_SnapPanels(BeneCast_SnapTo[snapto_option].AddOn);
	BeneCast_AttachPartyPanelsFromVar(); --WINTROW.6
	UIDropDownMenu_SetSelectedValue(BeneCastSnapToMenu, snapto_option);
	for i in pairs (BC_targets) do
		BeneCast_UpdateButtons(i);
	end
end

function PutDebugMsg(msg)
	if ( BeneCastSetup['Debug'] and DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage('Debug: ' .. msg);
	end
end

function PutMsg(msg)
	if ( DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage(msg);
	end
end

function BeneCastSnapToMenu_SetSnap()

	if ( not this.value ) then
		return;
	end
	local pf = BeneCastSetup.currentoptionid;
	if BeneCastProfiles[pf] == nil then
		BeneCastProfiles[pf] = {};
	end
	PutDebugMsg('Entered BeneCastSnapToMenu_SetSnap'); --WINTROW.6
	BeneCastProfiles[pf].SnapTo = this.value;
	if this.value ~= 'CUSTOM' then
		BeneCastProfiles[pf].SnapToBackup = this.value;
	end
	PutDebugMsg('SnapTo = ' .. this.value);
	BeneCast_SnapPanels(BeneCast_SnapTo[this.value].AddOn,true);
	BeneCast_AttachPartyPanelsFromVar();
	UIDropDownMenu_SetSelectedValue(BeneCastSnapToMenu, this.value);
	for i in pairs (BC_targets) do
		BeneCast_UpdateButtons(i);
	end
	PutDebugMsg('Updated buttons');

end

function BeneCastSnapToMenu_Load()

	if BeneCastSetup.currentoptionid == nil then
		BeneCastSetup.currentoptionid = 1;
	end
	local pf = BeneCastSetup.currentoptionid;
	if BeneCastProfiles[pf] == nil then
		BeneCastProfiles[pf] = {};
	end
	UIDropDownMenu_Initialize(this, BeneCastSnapToMenu_Initialize);
	UIDropDownMenu_SetSelectedValue(this, BeneCastProfiles[pf].SnapTo);
	getglobal('BeneCastSnapToMenuLabel'):SetText(BENECAST_STRINGS.TEXT_SNAP_TO);
	BeneCastSnapToMenu.tooltip = BENECAST_STRINGS.TEXT_SNAP_TO_TOOLTIP;
	UIDropDownMenu_SetWidth(110, BeneCastSnapToMenu);

end

-- *****************************************************************************
-- BeneCastRaidSnapToMenu functions
-- *****************************************************************************

function BeneCastRaidSnapToMenu_Initialize()
	
	local info;
	-- Add SnapTo options
	if BeneCastSetup.currentoptionid == nil then
		BeneCastSetup.currentoptionid = 1;
	end
	local pf = BeneCastSetup.currentoptionid;
	if BeneCastProfiles[pf] == nil then
		BeneCastProfiles[pf] = {};
	end
	for snapto in pairs (BeneCast_RaidSnapTo) do
		if ( IsAddOnLoaded(BeneCast_RaidSnapTo[snapto].AddOn) ) then
			info = {};
			if ( BeneCast_RaidSnapTo[snapto].AddOn == 'BeneCast' ) then
				info.text = BENECAST_STRINGS.TEXT_STANDARD;
			else
				info.text = BeneCast_RaidSnapTo[snapto].AddOn;
			end
			-- Determine the position for BeneCast panels to snap to
			if ( string.find(snapto, 'BOTTOM') ) then
				info.text = info.text .. BENECAST_STRINGS.TEXT_BOTTOM;
			elseif ( string.find(snapto, 'LEFT') ) then
				info.text = info.text .. BENECAST_STRINGS.TEXT_LEFT;
			elseif ( string.find(snapto, 'RIGHT') ) then
				info.text = info.text .. BENECAST_STRINGS.TEXT_RIGHT;
			elseif ( string.find(snapto, 'TOP') ) then
				info.text = info.text .. BENECAST_STRINGS.TEXT_TOP;
			end
			
			info.value = snapto;
			if ( BeneCastProfiles[pf].RaidSnapTo == snapto ) then
				info.checked = true;
			else
				info.checked = false;
			end
			info.func = BeneCastRaidSnapToMenu_SetSnap;
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
		end
	end

end

function BeneCastRaidSnapToMenu_SetSnap()

	if BeneCastSetup.currentoptionid == nil then
		BeneCastSetup.currentoptionid = 1;
	end
	local pf = BeneCastSetup.currentoptionid;
	if BeneCastProfiles[pf] == nil then
		BeneCastProfiles[pf] = {};
	end
	BeneCastProfiles[pf].RaidSnapTo = this.value;
	UIDropDownMenu_SetSelectedValue(BeneCastRaidSnapToMenu, this.value);
	BC_UpdateRaid = true;
	
end

function BeneCastRaidSnapToMenu_Load()

	if BeneCastSetup.currentoptionid == nil then
		BeneCastSetup.currentoptionid = 1;
	end
	local pf = BeneCastSetup.currentoptionid;
	if BeneCastProfiles[pf] == nil then
		BeneCastProfiles[pf] = {};
	end
	UIDropDownMenu_Initialize(this, BeneCastRaidSnapToMenu_Initialize);
	UIDropDownMenu_SetSelectedValue(this, BeneCastProfiles[pf].RaidSnapTo);
	getglobal('BeneCastRaidSnapToMenuLabel'):SetText(BENECAST_STRINGS.TEXT_RAID_SNAP_TO);
	BeneCastRaidSnapToMenu.tooltip = BENECAST_STRINGS.TEXT_RAID_SNAP_TO_TOOLTIP;

end

-- *****************************************************************************
-- BeneCastNotificationDropDown functions
-- *****************************************************************************

function BeneCastNotificationDropDown_Initialize()
	
	if BeneCastSetup.currentoptionid == nil then
		BeneCastSetup.currentoptionid = 1;
	end
	local pf = BeneCastSetup.currentoptionid;
	if BeneCastProfiles[pf] == nil then
		BeneCastProfiles[pf] = {};
	end

	local info, buttonIndex;

	-- Add the menu title which will be the default selected item.
	info = {};
	info.text = BENECAST_STRINGS.TEXT_USER_CHANNEL;
	info.value = 'Tip';
	info.isTitle = true;
	info.checked = false;
	info.owner = this;

	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

	-- Finds the index of the associated dropdown
	buttonIndex = this:GetID();
	if buttonIndex == 0 then
		buttonIndex = this:GetParent():GetID();
	end
	
	-- Finds the config of the current dropdown box
	local values = BeneCastOptionDropDowns[buttonIndex];
	if values == nil then
		PutDebugMsg('no BeneCastOptionDropDowns for id ' .. buttonIndex .. ' object ' .. this:GetName());
		return;
	end

	-- Add Available notification targets
	info = {};
	info.arg1 = {};
	for textReference, channel in pairs (BeneCastNotificationTargets) do
		info.text = BENECAST_STRINGS[textReference];
		info.value = channel;
		info.arg1.chan = channel;
		info.arg1.cvar = values.cvar;
		info.arg1.dropdown = this;
		info.arg1.index = buttonIndex;
		info.func = BeneCastNotificationDropDown_OnClick;
		info.tooltipTitle = BENECAST_STRINGS[textReference];
		info.tooltipText = BENECAST_STRINGS[textReference .. '_TOOLTIP'];
		info.checked = (BeneCastProfiles[pf][values.cvar] == channel);
		info.owner = this;
		
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
	end

end

function BeneCastNotificationDropDown_OnClick(arg1)

	if BeneCastSetup.currentoptionid == nil then
		BeneCastSetup.currentoptionid = 1;
	end
	local pf = BeneCastSetup.currentoptionid;
	if BeneCastProfiles[pf] == nil then
		BeneCastProfiles[pf] = {};
	end
	-- Finds the index of the associated dropdown
	PutDebugMsg('Clicked on ' .. arg1.dropdown:GetName() .. ' id=' .. arg1.index);
	if BeneCastOptionDropDowns[buttonIndex] then
		if BeneCastProfiles[pf].CascadeChannels then
			for i,values in pairs (BeneCastOptionDropDowns) do
				BeneCastProfiles[pf][values.cvar] = arg1.chan;
				UIDropDownMenu_SetSelectedValue(getglobal('BeneCastOptionDropDown' .. i),arg1.chan);
			end
		else
			BeneCastProfiles[pf][arg1.cvar] = arg1.chan;
			UIDropDownMenu_SetSelectedValue(arg1.dropdown, arg1.chan);
		end
	end
	
end

function BeneCastNotificationDropDown_Load()

	if BeneCastSetup.currentoptionid == nil then
		BeneCastSetup.currentoptionid = 1;
	end
	local pf = BeneCastSetup.currentoptionid;
	if BeneCastProfiles[pf] == nil then
		BeneCastProfiles[pf] = {};
	end
	-- Finds the index of the associated dropdown
	local buttonIndex = this:GetID();
	if buttonIndex == 0 then
		buttonIndex = this:GetParent():GetID();
	end
	local cvar = BeneCastOptionDropDowns[buttonIndex].cvar;
	UIDropDownMenu_Initialize(this, BeneCastNotificationDropDown_Initialize);
	if not BeneCastProfiles[pf][cvar] then
		BeneCastProfiles[pf][cvar] = 'SELF';
	end
	UIDropDownMenu_SetSelectedValue(this,BeneCastProfiles[pf][cvar]);
end

-- *****************************************************************************
-- BeneCastOptionFrame functions
-- *****************************************************************************

function BeneCastOptionFrame_OnLoad()
	
	-- Binding labels
	BINDING_HEADER_BENECAST = 'BeneCast';
	BINDING_NAME_BENECAST_OPTIONS = BENECAST_STRINGS.TEXT_BENECAST_OPTIONS;
	BINDING_NAME_BENECAST_TOGGLE_BUFFS = BENECAST_STRINGS.TEXT_BENECAST_TOGGLE_BUFFS;
	BINDING_NAME_BENECAST_BUTTON1 = BENECAST_STRINGS.TEXT_BENECAST_BUTTON1;
	BINDING_NAME_BENECAST_BUTTON2 = BENECAST_STRINGS.TEXT_BENECAST_BUTTON2;
	BINDING_NAME_BENECAST_BUTTON3 = BENECAST_STRINGS.TEXT_BENECAST_BUTTON3;
	BINDING_NAME_BENECAST_BUTTON4 = BENECAST_STRINGS.TEXT_BENECAST_BUTTON4;
	BINDING_NAME_BENECAST_BUTTON5 = BENECAST_STRINGS.TEXT_BENECAST_BUTTON5;
	BINDING_NAME_BENECAST_BUTTON6 = BENECAST_STRINGS.TEXT_BENECAST_BUTTON6;
	BINDING_NAME_BENECAST_BUTTON7 = BENECAST_STRINGS.TEXT_BENECAST_BUTTON7;
	BINDING_NAME_BENECAST_BUTTON8 = BENECAST_STRINGS.TEXT_BENECAST_BUTTON8;
	BINDING_NAME_BENECAST_BUTTON9 = BENECAST_STRINGS.TEXT_BENECAST_BUTTON9;
	BINDING_NAME_BENECAST_BUTTON10 = BENECAST_STRINGS.TEXT_BENECAST_BUTTON10;
	SLASH_BENECAST1 = '/benecast';
	SLASH_BENECAST2 = '/bc';
	
	-- Setup the BeneCast Slash Command Handler
	SlashCmdList['BENECAST'] = function(msg)
		BeneCast_SlashCommandHandler(msg);
	end
	
	-- Set up the title in the option frame
	getglobal('BeneCastTitleText'):SetText(BENECAST_STRINGS.TEXT_TITLE);

	-- Check Tab 1 and show the Player spell selection subframe
	getglobal('BeneCastPlayerFrame'):Show();
	getglobal('BeneCastOptionFrameTab1'):SetChecked(1);
	getglobal('BeneCastOptionFrameTab1'):Disable();
	
	-- Setup Tab Text and Graphics
	getglobal('BeneCastOptionFrameTab1NormalTexture'):SetTexture('Interface\\Icons\\Spell_Nature_WispSplode');
	getglobal('BeneCastOptionFrameTab1').tooltipText = UnitName('player');
	
	getglobal('BeneCastOptionFrameTab2NormalTexture'):SetTexture('Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes');
	getglobal('BeneCastOptionFrameTab2NormalTexture'):SetTexCoord(0.7421875, 0.98828125, 0, 0.25);
	getglobal('BeneCastOptionFrameTab2').tooltipText = BENECAST_STRINGS.CLASS_DRUID;
	
	getglobal('BeneCastOptionFrameTab3NormalTexture'):SetTexture('Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes');
	getglobal('BeneCastOptionFrameTab3NormalTexture'):SetTexCoord(0, 0.25, 0.25, 0.5);
	getglobal('BeneCastOptionFrameTab3').tooltipText = BENECAST_STRINGS.CLASS_HUNTER;
	
	getglobal('BeneCastOptionFrameTab4NormalTexture'):SetTexture('Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes');
	getglobal('BeneCastOptionFrameTab4NormalTexture'):SetTexCoord(0.25, 0.49609375, 0, 0.25);
	getglobal('BeneCastOptionFrameTab4').tooltipText = BENECAST_STRINGS.CLASS_MAGE;
	
	getglobal('BeneCastOptionFrameTab5NormalTexture'):SetTexture('Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes');
	getglobal('BeneCastOptionFrameTab5NormalTexture'):SetTexCoord(0.49609375, 0.7421875, 0.25, 0.5);
	getglobal('BeneCastOptionFrameTab5').tooltipText = BENECAST_STRINGS.CLASS_PRIEST;
	
	getglobal('BeneCastOptionFrameTab6NormalTexture'):SetTexture('Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes');
	getglobal('BeneCastOptionFrameTab6NormalTexture'):SetTexCoord(0.49609375, 0.7421875, 0, 0.25);
	getglobal('BeneCastOptionFrameTab6').tooltipText = BENECAST_STRINGS.CLASS_ROGUE;
	
	getglobal('BeneCastOptionFrameTab7NormalTexture'):SetTexture('Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes');
	getglobal('BeneCastOptionFrameTab7NormalTexture'):SetTexCoord(0.7421875, 0.98828125, 0.25, 0.5);
	getglobal('BeneCastOptionFrameTab7').tooltipText = BENECAST_STRINGS.CLASS_WARLOCK;
	
	getglobal('BeneCastOptionFrameTab8NormalTexture'):SetTexture('Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes');
	getglobal('BeneCastOptionFrameTab8NormalTexture'):SetTexCoord(0, 0.25, 0, 0.25);
	getglobal('BeneCastOptionFrameTab8').tooltipText = BENECAST_STRINGS.CLASS_WARRIOR;

	getglobal('BeneCastOptionFrameTab9NormalTexture'):SetTexture('Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes');
	getglobal('BeneCastOptionFrameTab9NormalTexture'):SetTexCoord(0.25, 0.49609375, 0.25, 0.5);
	getglobal('BeneCastOptionFrameTab9').tooltipText = BENECAST_STRINGS.CLASS_SHAMAN;
	
	getglobal('BeneCastOptionFrameTab10NormalTexture'):SetTexture('Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes');
	getglobal('BeneCastOptionFrameTab10NormalTexture'):SetTexCoord(0.0, 0.25, 0.5, 0.75);
	getglobal('BeneCastOptionFrameTab10').tooltipText = BENECAST_STRINGS.CLASS_PALADIN;
	
	getglobal('BeneCastOptionFrameTab11NormalTexture'):SetTexture('Interface\\Icons\\INV_Misc_Wrench_01');
	getglobal('BeneCastOptionFrameTab11').tooltipText = BENECAST_STRINGS.TEXT_SETUP;
	
	getglobal('BeneCastOptionFrameTab12NormalTexture'):SetTexture('Interface\\Icons\\INV_Letter_08');
	getglobal('BeneCastOptionFrameTab12').tooltipText = BENECAST_STRINGS.TEXT_NOTIFICATION;
	
	getglobal('BeneCastOptionFrameTab13NormalTexture'):SetTexture('Interface\\Icons\\INV_Misc_Head_Dragon_01');
	getglobal('BeneCastOptionFrameTab13').tooltipText = BENECAST_STRINGS.TEXT_RAID;

	getglobal('BeneCastOptionFrameTab14NormalTexture'):SetTexture('Interface\\Icons\\INV_Misc_Wrench_02');
	getglobal('BeneCastOptionFrameTab14').tooltipText = 'Unit Frames';

end

-- *****************************************************************************
-- BeneCastMiniMapButton Game Event Handlers
-- *****************************************************************************

-- Clicks on the BeneCast button do different things
function BeneCastOptionFrameToggle()

	-- If the mouseButton is the left button toggle BeneCastFrame's visibility
	-- if ( arg1 == 'LeftButton' ) then
		local frame = getglobal('BeneCastOptionFrame');
		if ( frame:IsVisible() ) then
			frame:Hide();
		else
			frame:Show();
		end
	--end
	
end

function BeneCastMiniMap_OnMouseDown()

	if BeneCastSetup.currentid == nil then
		BeneCastSetup.currentid = 1;
	end
	local pf = BeneCastSetup.currentid;
	if BeneCastProfiles[pf] == nil then
		BeneCastProfiles[pf] = {};
	end
	if ( ( BeneCastProfiles[pf].Unlock ) and ( arg1 == 'LeftButton' ) ) then
		this:StartMoving();
		this.isMoving = true;
	end

end

function BeneCastMiniMap_OnLoad()

	this:RegisterEvent('ADDON_LOADED');

end

function BeneCastMiniMap_OnEvent()

	if ( event == 'ADDON_LOADED' and arg1 == 'BeneCast' ) then
		if BeneCastSetup.currentid == nil then
			BeneCastSetup.currentid = 1;
		end
		local pf = BeneCastSetup.currentid;
		if BeneCastProfiles[pf] == nil then
			BeneCastProfiles[pf] = {};
		end
		if ( BeneCastProfiles[pf].HideMinimap ) then
			getglobal('BeneCastMinimapButton'):Hide();
		end
	end

end
