local BeneCastPanel = BeneCast.Panel;

-- Function to set orientation of buttons
function BeneCastPanel:OrientButtons()
	
	if self.unit == nil then
		PutDebugMsg('BeneCast.Panel:OrientButtons(): unit is nil');
		return;
	end

	if self.pf then
		PutDebugMsg('BeneCast.Panel:OrientButtons(): pf is nil');
		return;
	end
	
	local profile, panelconf
	profile, panelconf = BeneCast:GetProfile(self.pf, self.unit);
	if profile == nil then
		return;
	end
	
	if not profile.Orientation then
		profile.Orientation = 'RIGHT';
	end
	
	if not panelconf.Orientation then
		panelconf.Orientation = profile.Orientation;
	end

	if InCombatLockdown() then
		return;
	end

	local button = self.next;
	local previousbutton = self;
	while button ~= nil do
		
		button:ClearAllPoints();
		if panelconf.Orientation == 'RIGHT' then
			button:SetPoint('LEFT',previousbutton,'RIGHT',3,0);
		elseif panelconf.Orientation == 'DOWN' then
			button:SetPoint('TOP',previousbutton,'BOTTOM',0,-3);
		elseif panelconf.Orientation == 'UP' then
			button:SetPoint('BOTTOM',previousbutton,'TOP',0,3);
		elseif panelconf.Orientation == 'LEFT' then
			button:SetPoint('RIGHT',previousbutton,'LEFT',-3,0);
		end
		
		previousbutton = button;
		button = button.next;
		
	end
	
	self:ResizeButtons();

end

-- Function to resize the buttons of one panel
function BeneCastPanel:ResizeButtons()

	if InCombatLockdown() then
		return;
	end
	
	if self.unit == nil then
		PutDebugMsg('BeneCast.Panel:OrientButtons(): unit is nil');
		return;
	end
	
	if self.pf then
		PutDebugMsg('BeneCast.Panel:OrientButtons(): pf is nil');
		return;
	end
	
	local profile = BeneCast:GetProfile(self.pf, self.unit);
	if profile == nil then
		return;
	end
	
	local buttonsize = profile.ButtonSize;
	local button = self.next;

	-- Resize the buttons based on the saved size for every button in a panel
	while button ~= nil do
	  button:Resize(buttonsize);
	  button = button.next;
	end

end

-- Function to create a new Panel based on the xml-template
-- Panel is stored in BeneCast.WorkVars.Panels[profile][unit]
function BeneCastPanel:NewPanel(pfid, unit)

	local panel = CreateFrame('Frame', 'BeneCastPanel_' .. pfid .. '_' .. unit, UIParent, 'BeneCastPanelTemplate');
	panel.OrientButtons = self.OrientButtons;
	panel.ResizeButtons = self.ResizeButtons;
	panel.pf = pfid;
	panel.unit = unit;
	panel:SetAttribute('unit',unit);
	panel:Hide();
	panel:SetFrameStrata('LOW');
	return panel;

end
